"""
test_argparser.py
*****************

Tests for src/utils/argparser.py
"""
import os
import glob
import sys
import tempfile
import shutil

from MTfit.extensions import default_cmd_opts
from MTfit.extensions import default_cmd_defaults
# from MTfit.extensions import default_tests
from MTfit.utilities.extensions import get_extensions
from MTfit.utilities.argparser import _ARGPARSE
from MTfit.utilities.argparser_defaults import DEFAULT_AMP_COLORMAP
from MTfit.utilities.unittest_utils import TestCase
from MTfit.utilities.argparser import MTfit_parser
from MTfit.utilities.argparser import get_MTfit_defaults
from MTfit.utilities.argparser import evaluate_extensions
from MTfit.utilities import argparser


class ParserTestCase(TestCase):

    def setUp(self):
        self.cwd = os.getcwd()
        if sys.version_info >= (3, 0):
            self.tempdir = tempfile.TemporaryDirectory()
            os.chdir(self.tempdir.name)
        else:
            self.tempdir = tempfile.mkdtemp()
            os.chdir(self.tempdir)
        out_files = glob.glob('*.out')
        log_files = glob.glob('*.log')
        mat_files = glob.glob('*.mat')
        inv_files = glob.glob('*.inv')
        mt_files = glob.glob('*.mt')
        hyp_files = glob.glob('*.hyp')
        self.old_files = {'out': out_files, 'log': log_files, 'mat': mat_files,
                          'inv': inv_files, 'mt': mt_files, 'hyp': hyp_files}
        self.keep = ['MTfit_build.log']

    def tearDown(self):
        for file_type in self.old_files.keys():
            for fname in glob.glob('*.'+file_type):
                if fname not in self.old_files[file_type] and fname not in self.keep:
                    try:
                        os.remove(fname)
                    except Exception:
                        pass
        try:
            os.remove('Test.inv')
        except Exception:
            pass
        try:
            os.remove('Test.scatangle')
        except Exception:
            pass
        try:
            os.remove('Test.p123')
        except Exception:
            pass
        try:
            os.remove('Test.p124')
        except Exception:
            pass
        try:
            os.remove('Test.i')
        except Exception:
            pass
        try:
            os.remove('Test2.i')
        except Exception:
            pass
        os.chdir(self.cwd)
        if sys.version_info >= (3, 0):
            self.tempdir.cleanup()
        else:
            try:
                shutil.rmtree(self.tempdir)
            except:
                pass

    def test_MTfit_parser(self):
        cmd_defaults = {}
        cmd_default_types = {}
        results = evaluate_extensions('MTfit.cmd_defaults', default_cmd_defaults)
        for result in results:
            cmd_defaults.update(result[0])
            cmd_default_types.update(result[1])
        defaults = get_MTfit_defaults(True, cmd_defaults, cmd_default_types)
        # extension_names, extension_test_plugins = get_extensions(
        #     'MTfit.tests', default_tests)
        extension_tests = []
        cmd_opt_names, cmd_opts = get_extensions(
            'MTfit.cmd_opts', default_cmd_opts)
        # for plugin_name in extension_names:
        #     if plugin_name in cmd_opt_names:
        #         try:
        #             extension_tests.append(
        #                 extension_test_plugins[plugin_name]()[2])
        #             test_names.append(plugin_name)
        #         except Exception:
        #             pass
        example_MTfit_script = """#!/bin/bash
        ##MTfit qsub script
        #PBS -S /bin/sh
        #PBS -N MTfit
        #PBS -l walltime=300:00:00
        #PBS -V
        #PBS -l nodes=16:ppn=8
        #PBS -l pmem=2Gb
        #PBS -q batch
        #PBS -m bae
        python -c "import MTfit; MTfit.__run__()" --number_location_samples=25 --inversionoptions="PPolarity,P/SHQRMSAmplitudeRatio,P/SVRQMSAmplitudeRatio" --compareconstrained --algorithm=time --anglescatterfilepath="""+os.path.abspath('./')+os.path.sep+"Test.scatangle --numberworkers=128 --time=36000.0 --datafile="+os.path.abspath('./')+os.path.sep+" --nstations=0\n"
        example_MTfit_script2 = """#!/bin/bash
        ##MTfit qsub script
        #PBS -S /bin/sh
        #PBS -N MTfit
        #PBS -l walltime=300:00:00
        #PBS -V
        #PBS -l nodes=16:ppn=8
        #PBS -l pmem=2Gb
        #PBS -q batch
        #PBS -m bae
        python -c "import MTfit; MTfit.__run__()" --number_location_samples=250 --inversionoptions="PPolarity,P/SHQRMSAmplitudeRatio,P/SVRQMSAmplitudeRatio" --algorithm=time --anglescatterfilepath="""+os.path.abspath('./')+os.path.sep+"Test.scatangle --numberworkers=128 --time=36000.0 --datafile="+os.path.abspath('./')+os.path.sep+" --nstations=0\n"

        try:
            os.remove('Test.i')
        except Exception:
            pass
        try:
            os.remove('Test2.i')
        except Exception:
            pass
        # data_file -d

        global _ARGPARSE
        _argparse = _ARGPARSE is True
        if _argparse:
            print('data_file -d check')
            with self.assertRaises(SystemExit):
                MTfit_parser(['test'], test=True)
            with self.assertRaises(SystemExit):
                MTfit_parser(['-d=Test'], test=True)
            with self.assertRaises(SystemExit):
                MTfit_parser(['--datafile=Test'], test=True)
            open('Test.i', 'w').write('test')
            options, options_map = MTfit_parser(['Test.i'], test=True)
            self.assertEqual(
                options['data_file'], [os.path.abspath('Test.i')])
            options, options_map = MTfit_parser(["-d=Test.i"], test=True)
            self.assertEqual(
                options['data_file'], [os.path.abspath('Test.i')])
            options, options_map = MTfit_parser(["-dTest.i"], test=True)
            self.assertEqual(
                options['data_file'], [os.path.abspath('Test.i')])
            options, options_map = MTfit_parser(
                ["--datafile=Test.i"], test=True)
            self.assertEqual(
                options['data_file'], [os.path.abspath('Test.i')])
            options, options_map = MTfit_parser(
                ["--datafile=*.i"], test=True)
            self.assertEqual(
                options['data_file'], [os.path.abspath('Test.i')])
            options, options_map = MTfit_parser(["--invext=i"], test=True)
            self.assertEqual(
                options['data_file'], [os.path.abspath('Test.i')])
            open('Test2.i', 'w').write('test')
            options, options_map = MTfit_parser(['Test.i'], test=True)
            self.assertEqual(
                options['data_file'], [os.path.abspath('Test.i')])
            options, options_map = MTfit_parser(['Test2.i'], test=True)
            self.assertEqual(
                options['data_file'], [os.path.abspath('Test2.i')])
            options, options_map = MTfit_parser(["-d=Test2.i"], test=True)
            self.assertEqual(
                options['data_file'], [os.path.abspath('Test2.i')])
            options, options_map = MTfit_parser(["-dTest.i"], test=True)
            self.assertEqual(
                options['data_file'], [os.path.abspath('Test.i')])
            options, options_map = MTfit_parser(
                ["--datafile=Test2.i"], test=True)
            self.assertEqual(
                options['data_file'], [os.path.abspath('Test2.i')])
            options, options_map = MTfit_parser(
                ["--datafile=*.i"], test=True)
            self.assertAlmostEqual(
                options['data_file'], [os.path.abspath('Test.i'), os.path.abspath('Test2.i')])
            options, options_map = MTfit_parser(["--invext=i"], test=True)
            self.assertAlmostEqual(
                options['data_file'], [os.path.abspath('Test.i'), os.path.abspath('Test2.i')])
            os.remove('Test2.i')
            # AngleScatterFile -s
            print('location_pdf_file -s check')
            options, options_map = MTfit_parser(
                ["-d=Test.i", "-s=*.i"], test=True)
            self.assertEqual(
                options['data_file'], [os.path.abspath('Test.i')])
            self.assertEqual(
                options['location_pdf_file_path'], [os.path.abspath('Test.i')])
            options, options_map = MTfit_parser(
                ["Test.i", "-s=*.i"], test=True)
            self.assertEqual(
                options['data_file'], [os.path.abspath('Test.i')])
            self.assertEqual(
                options['location_pdf_file_path'], [os.path.abspath('Test.i')])
            options, options_map = MTfit_parser(
                ["-s=*.i", 'Test.i'], test=True)
            self.assertEqual(
                options['data_file'], [os.path.abspath('Test.i')])
            self.assertEqual(
                options['location_pdf_file_path'], [os.path.abspath('Test.i')])
            # algorithm -a
            print('algorithm -a check')
            with self.assertRaises(SystemExit):
                MTfit_parser(["-a=iteration", 'Test.i'], test=True)
            options, options_map = MTfit_parser(
                ["-a=iterate", 'Test.i'], test=True)
            self.assertEqual(
                options['data_file'], [os.path.abspath('Test.i')])
            self.assertEqual(options['algorithm'], 'iterate')
            options, options_map = MTfit_parser(
                ["-a=time", 'Test.i'], test=True)
            self.assertEqual(
                options['data_file'], [os.path.abspath('Test.i')])
            self.assertEqual(options['algorithm'], 'time')
            # Parallel -l
            print('parallel -l check')
            self.assertTrue(
                options['parallel'] != defaults['single_threaded'])
            options, options_map = MTfit_parser(
                ["-l", 'Test.i'], test=True)
            self.assertFalse(options['parallel'])
            options, options_map = MTfit_parser(
                ["--single", 'Test.i'], test=True)
            self.assertFalse(options['parallel'])
            options, options_map = MTfit_parser(
                ["--singlethread", 'Test.i'], test=True)
            self.assertFalse(options['parallel'])
            self.assertFalse('singlethread' in options.keys())
            # Number of Workers -n
            print('number_workers -n check')
            self.assertEqual(options['n'], defaults['number_workers'])
            options, options_map = MTfit_parser(
                ["-n=1", 'Test.i'], test=True)
            self.assertEqual(options['n'], 1)
            with self.assertRaises(SystemExit):
                MTfit_parser(["-n=1.1", 'Test.i'], test=True)
            # PMem -m
            print('mem -m check')
            self.assertEqual(options['phy_mem'], defaults['memory'])
            options, options_map = MTfit_parser(
                ["-m=1.5", 'Test.i'], test=True)
            self.assertEqual(options['phy_mem'], 1.5)
            # DC -c
            print('dc -c check')
            self.assertTrue(options['dc'] == defaults['double-couple'])
            options, options_map = MTfit_parser(
                ["-c", 'Test.i'], test=True)
            self.assertTrue(options['dc'])
            # NStations
            print('--nstations check')
            self.assertTrue(
                options['number_stations'] == defaults['number_stations'])
            options, options_map = MTfit_parser(
                ["--nstations=20", 'Test.i'], test=True)
            self.assertEqual(options['number_stations'], 20)
            with self.assertRaises(SystemExit):
                MTfit_parser(["--nstations=20.2", 'Test.i'], test=True)
            # NAngleSamples
            print('--nanglesamples check')
            self.assertEqual(
                options['number_location_samples'], defaults['number_location_samples'])
            options, options_map = MTfit_parser(
                ["--nanglesamples=20", 'Test.i'], test=True)
            self.assertEqual(options['number_location_samples'], 20)
            options, options_map = MTfit_parser(
                ["--number_location_samples=20", 'Test.i'], test=True)
            self.assertEqual(options['number_location_samples'], 20)
            with self.assertRaises(SystemExit):
                MTfit_parser(["--nanglesamples=20.2", 'Test.i'], test=True)
            # inversion_options -i
            print('inversion_options -i check')
            if defaults['inversion_options']:
                self.assertTrue(options['inversion_options'] == defaults['inversion_options'].split(
                    ','), str(options['inversion_options'])+' '+str(defaults['inversion_options']))
            else:
                self.assertTrue(options['inversion_options'] == defaults['inversion_options'], str(
                    options['inversion_options'])+' '+str(defaults['inversion_options']))
            options, options_map = MTfit_parser(
                ["-i=PPolarity", 'Test.i'], test=True)
            self.assertEqual(options['inversion_options'], ['PPolarity'])
            options, options_map = MTfit_parser(
                ["--inversionoptions=PPolarity,PSHRMSAmplitudeRatio", 'Test.i'], test=True)
            self.assertEqual(
                options['inversion_options'], ['PPolarity', 'PSHRMSAmplitudeRatio'])
            # file_sampling check
            print('file_sampling -f check')
            self.assertTrue(
                options['file_sample'] == defaults['disk_sample'])
            options, options_map = MTfit_parser(
                ["-f", 'Test.i'], test=True)
            self.assertTrue(options['file_sample'])
            options, options_map = MTfit_parser(
                ["--file-sample", 'Test.i'], test=True)
            self.assertTrue(options['file_sample'])
            with self.assertRaises(SystemExit):
                MTfit_parser(
                    ["--file-sample", 'Test.i', '-a=mcmc'], test=True)
            # FID -o
            print('output file -o check')
            self.assertTrue(options['fid'] == defaults['output_file'])
            options, options_map = MTfit_parser(
                ["-o=XKCD", 'Test.i'], test=True)
            self.assertEqual(options['fid'], "XKCD")
            options, options_map = MTfit_parser(
                ["--fid=XKCD", 'Test.i'], test=True)
            self.assertEqual(options['fid'], "XKCD")
            # max_samples -x
            print('max_samples -x check')
            options, options_map = MTfit_parser(
                ["-a=iterate", 'Test.i'], test=True)
            options, options_map = MTfit_parser(
                ["-a=iterate", "-x=100", 'Test.i'], test=True)
            self.assertEqual(options['max_samples'], 100)
            options, options_map = MTfit_parser(
                ["-a=iterate", "--maxsamples=100", 'Test.i'], test=True)
            self.assertEqual(options['max_samples'], 100)
            with self.assertRaises(SystemExit):
                MTfit_parser(["-x=20.2", 'Test.i'], test=True)
            # max_time -t
            print('max_time -t check')
            options, options_map = MTfit_parser(['Test.i'], test=True)
            options, options_map = MTfit_parser(
                ["-a=iterate", 'Test.i'], test=True)
            with self.assertRaises(KeyError):
                options['max_time']
            options, options_map = MTfit_parser(
                ["-a=iterate", "-t=100", 'Test.i'], test=True)
            with self.assertRaises(KeyError):
                options['max_time']
            if defaults['algorithm'] != 'time':
                options, options_map = MTfit_parser(
                    ["-a=time", "-t=100", 'Test.i'], test=True)
            else:
                options, options_map = MTfit_parser(
                    ["-t=100", 'Test.i'], test=True)
            self.assertEqual(options['max_time'], 100)
            if defaults['algorithm'] != 'time':
                options, options_map = MTfit_parser(
                    ["-a=time", "--maxtime=100.1", 'Test.i'], test=True)
            else:
                options, options_map = MTfit_parser(
                    ["--maxtime=100.1", 'Test.i'], test=True)
            self.assertEqual(options['max_time'], 100.1)
            if defaults['algorithm'] != 'time':
                with self.assertRaises(SystemExit):
                    MTfit_parser(["-a=time", "-t=a", 'Test.i'], test=True)
            else:
                with self.assertRaises(SystemExit):
                    MTfit_parser(["-t=a", 'Test.i'], test=True)
            # multiple events
            print('multiple events test')
            options, options_map = MTfit_parser(
                ["-amcmc", "-e", 'Test.i'], test=True)
            self.assertTrue(options['multiple_events'])
            # relative amplitudes
            print('relative amplitudes test')
            options, options_map = MTfit_parser(
                ["-amcmc", "-r", 'Test.i'], test=True)
            self.assertTrue(options['relative_amplitude'])
            # marginalise relative test
            print('marginalise relative test')
            options, options_map = MTfit_parser(['Test.i'], test=True)
            self.assertFalse(options['marginalise_relative'])
            options, options_map = MTfit_parser(
                ["-amcmc", "-r", "--marginalise-relative", 'Test.i'], test=True)
            self.assertTrue(options['marginalise_relative'])
            with self.assertRaises(SystemExit):
                MTfit_parser(
                    ["-amcmc", "--marginalise-relative", 'Test.i'], test=True)
            # min number intersections
            print('minimum_number_intersections -S check')
            options, options_map = MTfit_parser(['Test.i'], test=True)
            self.assertEqual(options['minimum_number_intersections'], 2)
            options, options_map = MTfit_parser(
                ['Test.i', '-S1'], test=True)
            self.assertEqual(options['minimum_number_intersections'], 1)
            with self.assertRaises(SystemExit):
                MTfit_parser(["-S1", 'Test.i', '-r'], test=True)
            options, options_map = MTfit_parser(
                ['Test.i', '-S10', '-r'], test=True)
            self.assertEqual(options['minimum_number_intersections'], 10)
            # McMC options
            print('McMC options check')
            options, options_map = MTfit_parser(
                ["-a=mcmc", "-x=100", 'Test.i'], test=True)
            self.assertEqual(options['chain_length'], 100)
            self.assertEqual(
                options['dimension_jump_prob'], defaults['jump_probability'])
            self.assertEqual(
                options['min_acceptance_rate'], defaults['mcmc_min_acceptance_rate'])
            self.assertEqual(
                options['max_acceptance_rate'], defaults['mcmc_max_acceptance_rate'])
            self.assertEqual(
                options['acceptance_rate_window'], defaults['acceptance_rate_window'])
            self.assertEqual(
                options['learning_length'], defaults['learning_length'])
            self.assertEqual(options['algorithm'], 'mcmc')
            with self.assertRaises(SystemExit):
                MTfit_parser(["-u=T"], test=True)
            with self.assertRaises(SystemExit):
                MTfit_parser(["-v=T"], test=True)
            with self.assertRaises(SystemExit):
                MTfit_parser(["-w=T"], test=True)
            with self.assertRaises(SystemExit):
                MTfit_parser(["-j=T"], test=True)
            with self.assertRaises(SystemExit):
                MTfit_parser(["-z=T"], test=True)
            with self.assertRaises(SystemExit):
                MTfit_parser(["-z=10.1"], test=True)
            options, options_map = MTfit_parser(
                ["-a=transdmcmc", "-u=0.1", "-v=0.8", "-w=5000", "-j=0.2", '-z=100', 'Test.i'], test=True)
            self.assertEqual(
                options['chain_length'], defaults['transdmcmc_chain_length'])
            self.assertEqual(options['algorithm'], 'transdmcmc')
            self.assertEqual(options['dimension_jump_prob'], 0.2)
            self.assertEqual(options['min_acceptance_rate'], 0.1)
            self.assertEqual(options['max_acceptance_rate'], 0.8)
            self.assertEqual(options['acceptance_rate_window'], 5000)
            self.assertEqual(options['learning_length'], 100)
            options, options_map = MTfit_parser(
                ["-a=transdmcmc", 'Test.i'], test=True)
            self.assertEqual(
                options['dimension_jump_prob'], defaults['jump_probability'])
            self.assertEqual(
                options['min_acceptance_rate'], defaults['transdmcmc_min_acceptance_rate'])
            self.assertEqual(
                options['max_acceptance_rate'], defaults['transdmcmc_max_acceptance_rate'])
            self.assertEqual(
                options['acceptance_rate_window'], defaults['acceptance_rate_window'])
            self.assertEqual(
                options['learning_length'], defaults['learning_length'])
            print('Warnings -W check')
            options, options_map = MTfit_parser(
                ["-W=d", 'Test.i'], test=True)
            self.assertEqual(options['warnings'], 'd')
            options, options_map = MTfit_parser(
                ["-W=ignore", 'Test.i'], test=True)
            self.assertEqual(options['warnings'], 'i')
            with self.assertRaises(SystemExit):
                MTfit_parser(["-WABC", 'Test.i'], test=True)
            options, options_map = MTfit_parser(['Test.i'], test=True)
            self.assertEqual(options['warnings'], 'd')
            # min_number_initialisation_samples -X
            print("min_number_initialisation/check_samples -X check")
            options, options_map = MTfit_parser(['Test.i'], test=True)
            self.assertEqual(
                options['min_number_check_samples'], defaults['min_number_check_samples'])
            options, options_map = MTfit_parser(
                ['Test.i', '-X=1000'], test=True)
            self.assertEqual(options['min_number_check_samples'], 1000)
            options, options_map = MTfit_parser(
                ['Test.i', "-amcmc"], test=True)
            self.assertEqual(options['min_number_initialisation_samples'], defaults[
                             'mcmc_min_number_initialisation_samples'])
            options, options_map = MTfit_parser(
                ['Test.i', "-amcmc", '-X=1000'], test=True)
            self.assertEqual(
                options['min_number_initialisation_samples'], 1000)
            # debug
            print('debug -D check')
            options, options_map = MTfit_parser(['Test.i'], test=True)
            self.assertTrue(options['debug'] == defaults['debug'])
            options, options_map = MTfit_parser(
                ['Test.i', '-D'], test=True)
            self.assertTrue(options['debug'])
            with self.assertRaises(SystemExit):
                MTfit_parser(["-D", 'Test.i', '-q'], test=True)
            # benchmark check
            print('benchmark -B check')
            options, options_map = MTfit_parser(['Test.i'], test=True)
            self.assertTrue(options['benchmark'] == defaults['benchmark'])
            options, options_map = MTfit_parser(
                ['Test.i', '-B'], test=True)
            self.assertTrue(options['benchmark'])
            # quality_check
            print('quality -Q check')
            options, options_map = MTfit_parser(['Test.i'], test=True)
            self.assertTrue(
                options['quality_check'] == defaults['quality'])
            options, options_map = MTfit_parser(
                ['Test.i', '-Q'], test=True)
            self.assertEqual(
                options['quality_check'], defaults['quality_check_value'])
            options, options_map = MTfit_parser(
                ['Test.i', '-Q=1'], test=True)
            self.assertEqual(options['quality_check'], 1)
            options, options_map = MTfit_parser(
                ['Test.i', '-Q=1', '-o=XKCD'], test=True)
            self.assertEqual(options['fid'], "XKCD")
            self.assertEqual(options['quality_check'], 1)
            # output-format
            options, options_map = MTfit_parser(['Test.i'], test=True)
            self.assertTrue(
                options['output_format'] == defaults['output_format'])
            options, options_map = MTfit_parser(
                ['--output-format=matlab', 'Test.i'], test=True)
            self.assertEqual(options['output_format'], 'matlab')
            options, options_map = MTfit_parser(
                ['--output-format=pickle', 'Test.i'], test=True)
            self.assertEqual(options['output_format'], 'pickle')
            with self.assertRaises(SystemExit):
                MTfit_parser(
                    ['--output-format=goadasdafqfeqwjfoijoidfawgohgkajrghaioarhgnwae', 'Test.i'], test=True)
            # results-format
            options, options_map = MTfit_parser(['Test.i'], test=True)
            self.assertTrue(
                options['results_format'] == defaults['results_format'])
            options, options_map = MTfit_parser(
                ['--results-format=full_pdf', 'Test.i'], test=True)
            self.assertEqual(options['results_format'], 'full_pdf')
            with self.assertRaises(SystemExit):
                MTfit_parser(
                    ['--results-format=goadasdafqfeqwjfoijoidfawgohgkajrghaioarhgnwae', 'Test.i'], test=True)
            # dc_prior
            print('dc_prior test')
            options, options_map = MTfit_parser(['Test.i'], test=True)
            self.assertTrue(options['dc_prior'] == defaults['dc_prior'])
            options, options_map = MTfit_parser(
                ['Test.i', '--dc_prior=0.5'], test=True)
            self.assertEqual(options['dc_prior'], 0.5)
            with self.assertRaises(SystemExit):
                MTfit_parser(
                    ['Test.i', '-a=transdmcmc', '--dc_prior=1.5'], test=True)
            # normalise
            print('normalise test')
            options, options_map = MTfit_parser(['Test.i'], test=True)
            self.assertTrue(
                options['normalise'] != defaults['no_normalise'])
            options, options_map = MTfit_parser(
                ['Test.i', '--no_normalise'], test=True)
            self.assertEqual(options['normalise'], False)
            # convert
            print('convert test')
            options, options_map = MTfit_parser(['Test.i'], test=True)
            self.assertTrue(options['convert'] == defaults['convert'])
            options, options_map = MTfit_parser(
                ['Test.i', '--convert'], test=True)
            self.assertEqual(options['convert'], True)
            # discard
            print('discard test')
            options, options_map = MTfit_parser(['Test.i'], test=True)
            self.assertTrue(options['discard'] == defaults['discard'])
            options, options_map = MTfit_parser(
                ['Test.i', '--discard=23000'], test=True)
            self.assertEqual(options['discard'], 23000)
            # extensions:
            print('Testing extensions for command line parsers')
            for test in extension_tests:
                test(self, MTfit_parser, defaults, _ARGPARSE)
            # qsub
            open('Test.i', 'w').write('test')
            print('qsub -q check')
            options, options_map = MTfit_parser(['Test.i'], test=True)
            self.assertFalse(options['qsub'])
            if 'PBS_DEFAULT' not in os.environ.keys() or 'PBS_NUM_NODES' in os.environ.keys():
                with self.assertRaises(SystemExit):
                    MTfit_parser(['Test.i', '-q'], test=True)
            open('Test.inv', 'w').write('test')
            open('Test.scatangle', 'w').write('test')
            argparser._QSUBTEST = True
            if argparser._PYQSUB:
                open('Test.p123', 'w').write(example_MTfit_script)
                options, options_map = MTfit_parser(['-R', 'Test.p123'], test=True)
                self.assertTrue(options['recover'])
                self.assertEqual(options['number_location_samples'], 25)
                self.assertTrue(options['dc_mt'])
                open('Test.p124', 'w').write(example_MTfit_script2)
                options, options_map = MTfit_parser(['-R'], test=True)
                self.assertTrue(options['recover'])
                self.assertEqual(options['number_location_samples'], 250)
                self.assertFalse(options['dc_mt'])
            argparser._QSUBTEST = False
            options, options_map = MTfit_parser(['-R'], test=True)
            self.assertTrue(options['recover'])
            try:
                os.remove('Test.inv')
            except Exception:
                pass
            try:
                os.remove('Test.scatangle')
            except Exception:
                pass
            try:
                os.remove('Test.p123')
            except Exception:
                pass
            try:
                os.remove('Test.p124')
            except Exception:
                pass
            _ARGPARSE = False
            if argparser._PYQSUB:
                argparser.pyqsub.ARGPARSE = False
        if not _ARGPARSE:
            try:
                os.remove('Test.i')
            except Exception:
                pass
            try:
                os.remove('Test2.i')
            except Exception:
                pass
            print("\n\n------------OPTPARSE----------------\n\n")
            # data_file -d
            print('data_file -d check')
            with self.assertRaises(SystemExit):
                MTfit_parser(['test'], test=True)
            with self.assertRaises(SystemExit):
                MTfit_parser(['-dTest'], test=True)
            with self.assertRaises(SystemExit):
                MTfit_parser(['--datafile=Test'], test=True)
            open('Test.i', 'w').write('test')
            options, options_map = MTfit_parser(['Test.i'], test=True)
            self.assertEqual(
                options['data_file'], [os.path.abspath('Test.i')])
            options, options_map = MTfit_parser(["-dTest.i"], test=True)
            self.assertEqual(
                options['data_file'], [os.path.abspath('Test.i')])
            options, options_map = MTfit_parser(["-dTest.i"], test=True)
            self.assertEqual(
                options['data_file'], [os.path.abspath('Test.i')])
            options, options_map = MTfit_parser(
                ["--datafile=Test.i"], test=True)
            self.assertEqual(options['data_file'], [os.path.abspath('Test.i')])
            options, options_map = MTfit_parser(
                ["--datafile=*.i"], test=True)
            self.assertEqual(
                options['data_file'], [os.path.abspath('Test.i')])
            options, options_map = MTfit_parser(["--invext=i"], test=True)
            self.assertEqual(
                options['data_file'], [os.path.abspath('Test.i')])
            open('Test2.i', 'w').write('test')
            options, options_map = MTfit_parser(['Test.i'], test=True)
            self.assertEqual(
                options['data_file'], [os.path.abspath('Test.i')])
            options, options_map = MTfit_parser(['Test2.i'], test=True)
            self.assertEqual(
                options['data_file'], [os.path.abspath('Test2.i')])
            options, options_map = MTfit_parser(["-dTest2.i"], test=True)
            self.assertEqual(
                options['data_file'], [os.path.abspath('Test2.i')])
            options, options_map = MTfit_parser(["-dTest.i"], test=True)
            self.assertEqual(
                options['data_file'], [os.path.abspath('Test.i')])
            options, options_map = MTfit_parser(
                ["--datafile=Test2.i"], test=True)
            self.assertEqual(
                options['data_file'], [os.path.abspath('Test2.i')])
            options, options_map = MTfit_parser(
                ["--datafile=*.i"], test=True)
            self.assertAlmostEqual(
                options['data_file'], [os.path.abspath('Test.i'), os.path.abspath('Test2.i')])
            options, options_map = MTfit_parser(["--invext=i"], test=True)
            self.assertAlmostEqual(
                options['data_file'], [os.path.abspath('Test.i'), os.path.abspath('Test2.i')])
            os.remove('Test2.i')
            # AngleScatterFile -s
            print('location_pdf_file -s check')
            options, options_map = MTfit_parser(
                ["-dTest.i", "-s*.i"], test=True)
            self.assertEqual(
                options['data_file'], [os.path.abspath('Test.i')])
            self.assertEqual(
                options['location_pdf_file_path'], [os.path.abspath('Test.i')])
            options, options_map = MTfit_parser(
                ["Test.i", "-s*.i"], test=True)
            self.assertEqual(
                options['data_file'], [os.path.abspath('Test.i')])
            self.assertEqual(
                options['location_pdf_file_path'], [os.path.abspath('Test.i')])
            options, options_map = MTfit_parser(
                ["-s*.i", 'Test.i'], test=True)
            self.assertEqual(
                options['data_file'], [os.path.abspath('Test.i')])
            self.assertEqual(
                options['location_pdf_file_path'], [os.path.abspath('Test.i')])
            # algorithm -a
            print('algorithm -a check')
            with self.assertRaises(SystemExit):
                MTfit_parser(["-a=iteration", 'Test.i'], test=True)
            options, options_map = MTfit_parser(
                ["-aiterate", 'Test.i'], test=True)
            self.assertEqual(
                options['data_file'], [os.path.abspath('Test.i')])
            self.assertEqual(options['algorithm'], 'iterate')
            options, options_map = MTfit_parser(
                ["-atime", 'Test.i'], test=True)
            self.assertEqual(
                options['data_file'], [os.path.abspath('Test.i')])
            self.assertEqual(options['algorithm'], 'time')
            # Parallel -l
            print('parallel -l check')
            self.assertTrue(
                options['parallel'] != defaults['single_threaded'])
            options, options_map = MTfit_parser(
                ["-l", 'Test.i'], test=True)
            self.assertFalse(options['parallel'])
            options, options_map = MTfit_parser(
                ["--single", 'Test.i'], test=True)
            self.assertFalse(options['parallel'])
            options, options_map = MTfit_parser(
                ["--singlethread", 'Test.i'], test=True)
            self.assertFalse(options['parallel'])
            self.assertFalse('singlethread' in options.keys())
            # Number of Workers -n
            print('number_workers -n check')
            self.assertEqual(options['n'], defaults['number_workers'])
            options, options_map = MTfit_parser(
                ["-n1", 'Test.i'], test=True)
            self.assertEqual(options['n'], 1)
            with self.assertRaises(SystemExit):
                MTfit_parser(["-n=1.1", 'Test.i'], test=True)
            # PMem -m
            print('mem -m check')
            self.assertEqual(options['phy_mem'], defaults['memory'])
            options, options_map = MTfit_parser(
                ["-m1.5", 'Test.i'], test=True)
            self.assertEqual(options['phy_mem'], 1.5)
            # DC -c
            print('dc -c check')
            self.assertTrue(options['dc'] == defaults['double-couple'])
            options, options_map = MTfit_parser(
                ["-c", 'Test.i'], test=True)
            self.assertTrue(options['dc'])
            # NStations
            print('--nstations check')
            self.assertTrue(
                options['number_stations'] == defaults['number_stations'])
            options, options_map = MTfit_parser(
                ["--nstations=20", 'Test.i'], test=True)
            self.assertEqual(options['number_stations'], 20)
            with self.assertRaises(SystemExit):
                MTfit_parser(["--nstations=20.2", 'Test.i'], test=True)
            # NAngleSamples
            print('--nanglesamples check')
            self.assertEqual(
                options['number_location_samples'], defaults['number_location_samples'])
            options, options_map = MTfit_parser(
                ["--nanglesamples=20", 'Test.i'], test=True)
            self.assertEqual(options['number_location_samples'], 20)
            options, options_map = MTfit_parser(
                ["--number_location_samples=20", 'Test.i'], test=True)
            self.assertEqual(options['number_location_samples'], 20)
            with self.assertRaises(SystemExit):
                MTfit_parser(["--nanglesamples=20.2", 'Test.i'], test=True)
            # inversion_options -i
            print('inversion_options -i check')
            if defaults['inversion_options']:
                self.assertTrue(options['inversion_options'] == defaults['inversion_options'].split(
                    ','), str(options['inversion_options'])+' '+str(defaults['inversion_options']))
            else:
                self.assertTrue(options['inversion_options'] == defaults['inversion_options'], str(
                    options['inversion_options'])+' '+str(defaults['inversion_options']))
            options, options_map = MTfit_parser(
                ["-i=PPolarity", 'Test.i'], test=True)
            options, options_map = MTfit_parser(
                ["-iPPolarity", 'Test.i'], test=True)
            self.assertEqual(options['inversion_options'], ['PPolarity'])
            options, options_map = MTfit_parser(
                ["--inversionoptions=PPolarity,PSHRMSAmplitudeRatio", 'Test.i'], test=True)
            self.assertEqual(
                options['inversion_options'], ['PPolarity', 'PSHRMSAmplitudeRatio'])
            # file_sampling check
            print('file_sampling -f check')
            self.assertTrue(
                options['file_sample'] == defaults['disk_sample'])
            options, options_map = MTfit_parser(
                ["-f", 'Test.i'], test=True)
            self.assertTrue(options['file_sample'])
            options, options_map = MTfit_parser(
                ["--file-sample", 'Test.i'], test=True)
            self.assertTrue(options['file_sample'])
            with self.assertRaises(SystemExit):
                MTfit_parser(
                    ["--file-sample", 'Test.i', '-a=mcmc'], test=True)
            # FID -o
            print('output file -o check')
            self.assertTrue(options['fid'] == defaults['output_file'])
            options, options_map = MTfit_parser(
                ["-oXKCD", 'Test.i'], test=True)
            self.assertEqual(options['fid'], "XKCD")
            options, options_map = MTfit_parser(
                ["--fid=XKCD", 'Test.i'], test=True)
            self.assertEqual(options['fid'], "XKCD")
            # max_samples -x
            print('max_samples -x check')
            with self.assertRaises(KeyError):
                options['max_samples']
            options, options_map = MTfit_parser(
                ["-aiterate", 'Test.i'], test=True)
            options, options_map = MTfit_parser(
                ["-aiterate", "-x100", 'Test.i'], test=True)
            self.assertEqual(options['max_samples'], 100)
            options, options_map = MTfit_parser(
                ["-aiterate", "--maxsamples=100", 'Test.i'], test=True)
            self.assertEqual(options['max_samples'], 100)
            with self.assertRaises(SystemExit):
                MTfit_parser(["-x20.2", 'Test.i'], test=True)
            # max_time -t
            print('max_time -t check')
            options, options_map = MTfit_parser(['Test.i'], test=True)
            options, options_map = MTfit_parser(
                ["-aiterate", 'Test.i'], test=True)
            with self.assertRaises(KeyError):
                options['max_time']
            options, options_map = MTfit_parser(
                ["-aiterate", "-t100", 'Test.i'], test=True)
            with self.assertRaises(KeyError):
                options['max_time']
            if defaults['algorithm'] != 'time':
                options, options_map = MTfit_parser(
                    ["-a=time", "-t100", 'Test.i'], test=True)
            else:
                options, options_map = MTfit_parser(
                    ["-t100", 'Test.i'], test=True)
            self.assertEqual(options['max_time'], 100)
            if defaults['algorithm'] != 'time':
                options, options_map = MTfit_parser(
                    ["-a=time", "--maxtime=100.1", 'Test.i'], test=True)
            else:
                options, options_map = MTfit_parser(
                    ["--maxtime=100.1", 'Test.i'], test=True)
            self.assertEqual(options['max_time'], 100.1)
            if defaults['algorithm'] != 'time':
                with self.assertRaises(SystemExit):
                    MTfit_parser(["-a=time", "-ta", 'Test.i'], test=True)
            else:
                with self.assertRaises(SystemExit):
                    MTfit_parser(["-ta", 'Test.i'], test=True)
            # multiple events
            print('multiple events test')
            options, options_map = MTfit_parser(
                ["-amcmc", "-e", 'Test.i'], test=True)
            self.assertTrue(options['multiple_events'])
            # relative amplitudes
            print('relative amplitudes test')
            options, options_map = MTfit_parser(
                ["-amcmc", "-r", 'Test.i'], test=True)
            self.assertTrue(options['relative_amplitude'])
            # marginalise relative test
            print('marginalise relative test')
            options, options_map = MTfit_parser(['Test.i'], test=True)
            self.assertFalse(options['marginalise_relative'])
            options, options_map = MTfit_parser(
                ["-amcmc", "-r", "--marginalise-relative", 'Test.i'], test=True)
            self.assertTrue(options['marginalise_relative'])
            with self.assertRaises(SystemExit):
                MTfit_parser(
                    ["-amcmc", "--marginalise-relative", 'Test.i'], test=True)
            # min number intersections
            print('minimum_number_intersections -S check')
            options, options_map = MTfit_parser(['Test.i'], test=True)
            self.assertEqual(options['minimum_number_intersections'], 2)
            options, options_map = MTfit_parser(
                ['Test.i', '-S1'], test=True)
            self.assertEqual(options['minimum_number_intersections'], 1)
            with self.assertRaises(SystemExit):
                MTfit_parser(["-S1", 'Test.i', '-r'])
            options, options_map = MTfit_parser(
                ['Test.i', '-S10', '-r'], test=True)
            self.assertEqual(options['minimum_number_intersections'], 10)
            # McMC options
            print('McMC options check')
            options, options_map = MTfit_parser(
                ["-amcmc", "-x100", 'Test.i'], test=True)
            self.assertEqual(options['chain_length'], 100)
            self.assertEqual(
                options['dimension_jump_prob'], defaults['jump_probability'])
            self.assertEqual(
                options['min_acceptance_rate'], defaults['mcmc_min_acceptance_rate'])
            self.assertEqual(
                options['max_acceptance_rate'], defaults['mcmc_max_acceptance_rate'])
            self.assertEqual(
                options['acceptance_rate_window'], defaults['acceptance_rate_window'])
            self.assertEqual(
                options['learning_length'], defaults['learning_length'])
            self.assertEqual(options['algorithm'], 'mcmc')
            with self.assertRaises(SystemExit):
                MTfit_parser(["-u=T"], test=True)
            with self.assertRaises(SystemExit):
                MTfit_parser(["-v=T"], test=True)
            with self.assertRaises(SystemExit):
                MTfit_parser(["-w=T"], test=True)
            with self.assertRaises(SystemExit):
                MTfit_parser(["-j=T"], test=True)
            with self.assertRaises(SystemExit):
                MTfit_parser(["-z=T"], test=True)
            with self.assertRaises(SystemExit):
                MTfit_parser(["-z=10.1"], test=True)
            options, options_map = MTfit_parser(
                ["-atransdmcmc", "-u0.1", "-v0.8", "-w5000", "-j0.2", '-z100', 'Test.i'], test=True)
            self.assertEqual(
                options['chain_length'], defaults['transdmcmc_chain_length'])
            self.assertEqual(options['algorithm'], 'transdmcmc')
            self.assertEqual(options['dimension_jump_prob'], 0.2)
            self.assertEqual(options['min_acceptance_rate'], 0.1)
            self.assertEqual(options['max_acceptance_rate'], 0.8)
            self.assertEqual(options['acceptance_rate_window'], 5000)
            self.assertEqual(options['learning_length'], 100)
            options, options_map = MTfit_parser(
                ["-atransdmcmc", 'Test.i'], test=True)
            self.assertEqual(
                options['dimension_jump_prob'], defaults['jump_probability'])
            self.assertEqual(
                options['min_acceptance_rate'], defaults['transdmcmc_min_acceptance_rate'])
            self.assertEqual(
                options['max_acceptance_rate'], defaults['transdmcmc_max_acceptance_rate'])
            self.assertEqual(
                options['acceptance_rate_window'], defaults['acceptance_rate_window'])
            self.assertEqual(
                options['learning_length'], defaults['learning_length'])
            print('Warnings -W check')
            options, options_map = MTfit_parser(
                ["-Wd", 'Test.i'], test=True)
            self.assertEqual(options['warnings'], 'd')
            options, options_map = MTfit_parser(
                ["-Wignore", 'Test.i'], test=True)
            self.assertEqual(options['warnings'], 'i')
            with self.assertRaises(SystemExit):
                MTfit_parser(["-WABC", 'Test.i'], test=True)
            options, options_map = MTfit_parser(['Test.i'], test=True)
            self.assertEqual(options['warnings'], 'd')
            # min_number_initialisation_samples -X
            print("min_number_initialisation/check_samples -X check")
            options, options_map = MTfit_parser(['Test.i'], test=True)
            self.assertEqual(
                options['min_number_check_samples'], defaults['min_number_check_samples'])
            options, options_map = MTfit_parser(
                ['Test.i', '-X1000'], test=True)
            self.assertEqual(options['min_number_check_samples'], 1000)
            options, options_map = MTfit_parser(
                ['Test.i', "-amcmc"], test=True)
            self.assertEqual(options['min_number_initialisation_samples'], defaults[
                             'mcmc_min_number_initialisation_samples'])
            options, options_map = MTfit_parser(
                ['Test.i', "-amcmc", '-X1000'], test=True)
            self.assertEqual(
                options['min_number_initialisation_samples'], 1000)
            # debug
            print('debug -D check')
            options, options_map = MTfit_parser(['Test.i'], test=True)
            self.assertTrue(options['debug'] == defaults['debug'])
            options, options_map = MTfit_parser(
                ['Test.i', '-D'], test=True)
            self.assertTrue(options['debug'])
            with self.assertRaises(SystemExit):
                MTfit_parser(["-D", 'Test.i', '-q'], test=True)
            # benchmark check
            print('benchmark -B check')
            options, options_map = MTfit_parser(['Test.i'], test=True)
            self.assertTrue(options['benchmark'] == defaults['benchmark'])
            options, options_map = MTfit_parser(
                ['Test.i', '-B'], test=True)
            self.assertTrue(options['benchmark'])
            # quality_check
            print('quality -Q check')
            options, options_map = MTfit_parser(['Test.i'], test=True)
            self.assertTrue(
                options['quality_check'] == defaults['quality'])
            options, options_map = MTfit_parser(
                ['Test.i', '-Q1'], test=True)
            self.assertEqual(options['quality_check'], 1)
            options, options_map = MTfit_parser(
                ['Test.i', '-Q1', '-oXKCD'], test=True)
            self.assertEqual(options['fid'], "XKCD")
            self.assertEqual(options['quality_check'], 1)
            # output-format
            options, options_map = MTfit_parser(['Test.i'], test=True)
            self.assertTrue(
                options['output_format'] == defaults['output_format'])
            options, options_map = MTfit_parser(
                ['--output-format=matlab', 'Test.i'], test=True)
            self.assertEqual(options['output_format'], 'matlab')
            options, options_map = MTfit_parser(
                ['--output-format=pickle', 'Test.i'], test=True)
            self.assertEqual(options['output_format'], 'pickle')
            # results-format
            options, options_map = MTfit_parser(['Test.i'], test=True)
            self.assertTrue(
                options['results_format'] == defaults['results_format'])
            options, options_map = MTfit_parser(
                ['--results-format=full_pdf', 'Test.i'], test=True)
            self.assertEqual(options['results_format'], 'full_pdf')
            with self.assertRaises(SystemExit):
                MTfit_parser(
                    ['--results-format=goadasdafqfeqwjfoijoidfawgohgkajrghaioarhgnwae', 'Test.i'], test=True)
            # dc_prior
            print('dc_prior test')
            options, options_map = MTfit_parser(['Test.i'], test=True)
            self.assertTrue(options['dc_prior'] == defaults['dc_prior'])
            options, options_map = MTfit_parser(
                ['Test.i', '--dc_prior=0.5'], test=True)
            self.assertEqual(options['dc_prior'], 0.5)
            with self.assertRaises(SystemExit):
                MTfit_parser(
                    ['Test.i', '-atransdmcmc', '--dc_prior=1.5'], test=True)
            # normalise
            print('normalise test')
            options, options_map = MTfit_parser(['Test.i'], test=True)
            self.assertTrue(
                options['normalise'] != defaults['no_normalise'])
            options, options_map = MTfit_parser(
                ['Test.i', '--no_normalise'], test=True)
            self.assertEqual(options['normalise'], False)
            # convert
            print('convert test')
            options, options_map = MTfit_parser(['Test.i'], test=True)
            self.assertTrue(options['convert'] == defaults['convert'])
            options, options_map = MTfit_parser(
                ['Test.i', '--convert'], test=True)
            self.assertEqual(options['convert'], True)
            # discard
            print('discard test')
            options, options_map = MTfit_parser(['Test.i'], test=True)
            self.assertTrue(options['discard'] == defaults['discard'])
            options, options_map = MTfit_parser(
                ['Test.i', '--discard=23000'], test=True)
            self.assertEqual(options['discard'], 23000)
            # extensions:
            print('Testing extensions for command line parsers')
            for test in extension_tests:
                test(self, MTfit_parser, defaults, _ARGPARSE)
            open('Test.i', 'w').write('test')
            # qsub
            print('qsub -q check')
            options, options_map = MTfit_parser(['Test.i'], test=True)
            self.assertFalse(options['qsub'])
            if 'PBS_DEFAULT' not in os.environ.keys() or 'PBS_NUM_NODES' in os.environ.keys():
                with self.assertRaises(SystemExit):
                    MTfit_parser(['Test.i', '-q'], test=True)
            os.remove('Test.i')
            open('Test.inv', 'w').write('test')
            open('Test.scatangle', 'w').write('test')
            argparser._QSUBTEST = True
            if argparser._PYQSUB:
                open('Test.p123', 'w').write(example_MTfit_script)
                options, options_map = MTfit_parser(
                    ['-R', 'Test.p123'], test=True)
                self.assertTrue(options['recover'])
                self.assertEqual(options['number_location_samples'], 25)
                self.assertTrue(options['dc_mt'])
                open('Test.p124', 'w').write(example_MTfit_script2)
                options, options_map = MTfit_parser(['-R'], test=True)
                self.assertTrue(options['recover'])
                self.assertEqual(options['number_location_samples'], 250)
                self.assertFalse(options['dc_mt'])
            argparser._QSUBTEST = False
            options, options_map = MTfit_parser(['-R'], test=True)
            self.assertTrue(options['recover'])
            try:
                os.remove('Test.inv')
            except Exception:
                pass
            try:
                os.remove('Test.scatangle')
            except Exception:
                pass
            try:
                os.remove('Test.p123')
            except Exception:
                pass
            try:
                os.remove('Test.p124')
            except Exception:
                pass
            _ARGPARSE = _argparse
            if argparser._PYQSUB:
                argparser.pyqsub.ARGPARSE = _argparse

    def test_MTplot_parser(self):
        from MTfit.utilities.argparser import MTplot_parser
        from MTfit.utilities.argparser import get_MTplot_defaults
        from MTfit.utilities.argparser import evaluate_extensions
        cmd_defaults = {}
        cmd_default_types = {}
        results = evaluate_extensions('MTfit.cmd_defaults', default_cmd_defaults)
        for result in results:
            cmd_defaults.update(result[0])
            cmd_default_types.update(result[1])
        defaults = get_MTplot_defaults(True, cmd_defaults, cmd_default_types)
        # data_file -d
        global _ARGPARSE
        _argparse = _ARGPARSE is True
        if _argparse:
            print('data_file -d check')
            self.assertEqual(
                MTplot_parser(['test'], test=True)['data_file'], 'test')
            self.assertEqual(
                MTplot_parser(['-d=Test'], test=True)['data_file'], 'Test')
            self.assertEqual(
                MTplot_parser(['--datafile=Test'], test=True)['data_file'], 'Test')
            print('plot-type check')
            self.assertEqual(
                MTplot_parser(['test'], test=True)['plot_type'], defaults['plot_type'])
            self.assertEqual(
                MTplot_parser(['test', '-p=hudson'], test=True)['plot_type'], 'hudson')
            self.assertEqual(MTplot_parser(
                ['test', '--plot_type=faultplane'], test=True)['plot_type'], 'faultplane')
            print('colormap check')
            self.assertEqual(
                MTplot_parser(['test'], test=True)['colormap'], DEFAULT_AMP_COLORMAP)
            self.assertEqual(
                MTplot_parser(['test', '-c=jet'], test=True)['colormap'], 'jet')
            self.assertEqual(
                MTplot_parser(['test', '--color_map=bwr'], test=True)['colormap'], 'bwr')
            print('fontsize check')
            self.assertEqual(
                MTplot_parser(['test'], test=True)['fontsize'], defaults['fontsize'])
            self.assertEqual(
                MTplot_parser(['test', '-f=12'], test=True)['fontsize'], 12)
            self.assertEqual(
                MTplot_parser(['test', '--fontsize=8'], test=True)['fontsize'], 8)
            print('linewidth check')
            self.assertEqual(
                MTplot_parser(['test'], test=True)['linewidth'], defaults['linewidth'])
            self.assertEqual(
                MTplot_parser(['test', '-l=12'], test=True)['linewidth'], 12)
            self.assertEqual(
                MTplot_parser(['test', '--linewidth=8'], test=True)['linewidth'], 8)
            print('text check')
            self.assertEqual(
                MTplot_parser(['test'], test=True)['text'], defaults['text'])
            self.assertEqual(
                MTplot_parser(['test', '-t'], test=True)['text'], True)
            self.assertEqual(
                MTplot_parser(['test', '--text'], test=True)['text'], True)
            print('resolution check')
            self.assertEqual(
                MTplot_parser(['test'], test=True)['resolution'], defaults['resolution'])
            self.assertEqual(
                MTplot_parser(['test', '-r=12'], test=True)['resolution'], 12)
            self.assertEqual(
                MTplot_parser(['test', '--resolution=8'], test=True)['resolution'], 8)
            with self.assertRaises(SystemExit):
                MTplot_parser(['test', '--resolution=8.4'], test=True)
            print('bins check')
            self.assertEqual(
                MTplot_parser(['test'], test=True)['bins'], defaults['bins'])
            self.assertEqual(
                MTplot_parser(['test', '-b=12'], test=True)['bins'], 12)
            self.assertEqual(
                MTplot_parser(['test', '--bins=8'], test=True)['bins'], 8)
            with self.assertRaises(SystemExit):
                MTplot_parser(['test', '--bins=8.4'], test=True)
            print('fault_plane check')
            self.assertEqual(
                MTplot_parser(['test'], test=True)['fault_plane'], defaults['fault_plane'])
            self.assertEqual(
                MTplot_parser(['test', '--fault_plane'], test=True)['fault_plane'], True)
            print('nodal_line check')
            self.assertEqual(
                MTplot_parser(['test'], test=True)['nodal_line'], defaults['nodal_line'])
            self.assertEqual(
                MTplot_parser(['test', '-n'], test=True)['nodal_line'], True)
            self.assertEqual(
                MTplot_parser(['test', '--nodal_line'], test=True)['nodal_line'], True)
            print('TNP check')
            self.assertEqual(
                MTplot_parser(['test'], test=True)['TNP'], defaults['TNP'])
            self.assertEqual(
                MTplot_parser(['test', '--pt'], test=True)['TNP'], True)
            self.assertEqual(
                MTplot_parser(['test', '--tnp'], test=True)['TNP'], True)
            print('markersize check')
            self.assertEqual(
                MTplot_parser(['test'], test=True)['markersize'], defaults['markersize'])
            self.assertEqual(
                MTplot_parser(['test', '--markersize=8'], test=True)['markersize'], 8)
            self.assertEqual(
                MTplot_parser(['test', '--markersize=8.4'], test=True)['markersize'], 8.4)
            print('station_markersize check')
            self.assertEqual(MTplot_parser(['test'], test=True)[
                             'station_markersize'], defaults['station_markersize'])
            self.assertEqual(MTplot_parser(
                ['test', '--station_markersize=8'], test=True)['station_markersize'], 8)
            self.assertEqual(MTplot_parser(
                ['test', '--station_markersize=8.4'], test=True)['station_markersize'], 8.4)
            print('show_max_likelihood check')
            self.assertEqual(MTplot_parser(['test'], test=True)[
                             'show_max_likelihood'], defaults['show_max_likelihood'])
            self.assertEqual(MTplot_parser(
                ['test', '--showmaxlikelihood'], test=True)['show_max_likelihood'], True)
            self.assertEqual(MTplot_parser(
                ['test', '--show_max_likelihood'], test=True)['show_max_likelihood'], True)
            print('show_mean check')
            self.assertEqual(
                MTplot_parser(['test'], test=True)['show_mean'], defaults['show_mean'])
            self.assertEqual(
                MTplot_parser(['test', '--showmean'], test=True)['show_mean'], True)
            self.assertEqual(
                MTplot_parser(['test', '--show_mean'], test=True)['show_mean'], True)
            print('grid_lines check')
            self.assertEqual(
                MTplot_parser(['test'], test=True)['grid_lines'], defaults['grid_lines'])
            self.assertEqual(
                MTplot_parser(['test', '--gridlines'], test=True)['grid_lines'], True)
            self.assertEqual(
                MTplot_parser(['test', '--grid_lines'], test=True)['grid_lines'], True)
            print('color check')
            self.assertEqual(
                MTplot_parser(['test'], test=True)['color'], defaults['color'])
            self.assertEqual(
                MTplot_parser(['test', '--color=b'], test=True)['color'], 'b')
            self.assertEqual(
                MTplot_parser(['test', '--color=purple'], test=True)['color'], 'purple')
            print('type_label check')
            self.assertEqual(
                MTplot_parser(['test'], test=True)['type_label'], defaults['type_label'])
            self.assertEqual(
                MTplot_parser(['test', '--typelabel'], test=True)['type_label'], True)
            self.assertEqual(
                MTplot_parser(['test', '--type_label'], test=True)['type_label'], True)
            print('hex_bin check')
            self.assertEqual(
                MTplot_parser(['test'], test=True)['hex_bin'], defaults['hex_bin'])
            self.assertEqual(
                MTplot_parser(['test', '--hexbin'], test=True)['hex_bin'], True)
            self.assertEqual(
                MTplot_parser(['test', '--hex_bin'], test=True)['hex_bin'], True)
            print('projection check')
            self.assertEqual(
                MTplot_parser(['test'], test=True)['projection'], defaults['projection'])
            self.assertEqual(
                MTplot_parser(['test', '--projection=uv'], test=True)['projection'], 'uv')
            self.assertEqual(MTplot_parser(
                ['test', '--projection=equalangle'], test=True)['projection'], 'equalangle')
            print('save_file check')
            self.assertEqual(
                MTplot_parser(['test'], test=True)['save_file'], '')
            self.assertEqual(MTplot_parser(
                ['test', '--save_file=test.png'], test=True)['save_file'], 'test.png')
            print('save_dpi check')
            self.assertEqual(
                MTplot_parser(['test'], test=True)['save_dpi'], 200)
            self.assertEqual(
                MTplot_parser(['test', '--save_dpi=150'], test=True)['save_dpi'], 150)
            print('hide')
            self.assertEqual(
                MTplot_parser(['test'], test=True)['show'], True)
            self.assertEqual(
                MTplot_parser(['test', '-q'], test=True)['show'], False)
            self.assertEqual(
                MTplot_parser(['test', '--quiet'], test=True)['show'], False)
            self.assertEqual(
                MTplot_parser(['test', '--hide'], test=True)['show'], False)
            _ARGPARSE = False
        if not _ARGPARSE:
            print('data_file -d check')
            self.assertEqual(
                MTplot_parser(['test'], test=True)['data_file'], 'test')
            self.assertEqual(
                MTplot_parser(['-dTest'], test=True)['data_file'], 'Test')
            self.assertEqual(
                MTplot_parser(['--datafile=Test'], test=True)['data_file'], 'Test')
            print('plot-type check')
            self.assertEqual(
                MTplot_parser(['test'], test=True)['plot_type'], defaults['plot_type'])
            self.assertEqual(
                MTplot_parser(['test', '-phudson'], test=True)['plot_type'], 'hudson')
            self.assertEqual(MTplot_parser(
                ['test', '--plot_type=faultplane'], test=True)['plot_type'], 'faultplane')
            print('colormap check')
            self.assertEqual(
                MTplot_parser(['test'], test=True)['colormap'], DEFAULT_AMP_COLORMAP)
            self.assertEqual(
                MTplot_parser(['test', '-cjet'], test=True)['colormap'], 'jet')
            self.assertEqual(
                MTplot_parser(['test', '--color_map=bwr'], test=True)['colormap'], 'bwr')
            print('fontsize check')
            self.assertEqual(
                MTplot_parser(['test'], test=True)['fontsize'], defaults['fontsize'])
            self.assertEqual(
                MTplot_parser(['test', '-f12'], test=True)['fontsize'], 12)
            self.assertEqual(
                MTplot_parser(['test', '--fontsize=8'], test=True)['fontsize'], 8)
            print('linewidth check')
            self.assertEqual(
                MTplot_parser(['test'], test=True)['linewidth'], defaults['linewidth'])
            self.assertEqual(
                MTplot_parser(['test', '-l12'], test=True)['linewidth'], 12)
            self.assertEqual(
                MTplot_parser(['test', '--linewidth=8'], test=True)['linewidth'], 8)
            print('text check')
            self.assertEqual(
                MTplot_parser(['test'], test=True)['text'], defaults['text'])
            self.assertEqual(
                MTplot_parser(['test', '-t'], test=True)['text'], True)
            self.assertEqual(
                MTplot_parser(['test', '--text'], test=True)['text'], True)
            print('resolution check')
            self.assertEqual(
                MTplot_parser(['test'], test=True)['resolution'], defaults['resolution'])
            self.assertEqual(
                MTplot_parser(['test', '-r12'], test=True)['resolution'], 12)
            self.assertEqual(
                MTplot_parser(['test', '--resolution=8'], test=True)['resolution'], 8)
            with self.assertRaises(SystemExit):
                MTplot_parser(['test', '--resolution=8.4'], test=True)
            print('bins check')
            self.assertEqual(
                MTplot_parser(['test'], test=True)['bins'], defaults['bins'])
            self.assertEqual(
                MTplot_parser(['test', '-b12'], test=True)['bins'], 12)
            self.assertEqual(
                MTplot_parser(['test', '--bins=8'], test=True)['bins'], 8)
            with self.assertRaises(SystemExit):
                MTplot_parser(['test', '--bins=8.4'], test=True)
            print('fault_plane check')
            self.assertEqual(
                MTplot_parser(['test'], test=True)['fault_plane'], defaults['fault_plane'])
            self.assertEqual(
                MTplot_parser(['test', '--fault_plane'], test=True)['fault_plane'], True)
            print('nodal_line check')
            self.assertEqual(
                MTplot_parser(['test'], test=True)['nodal_line'], defaults['nodal_line'])
            self.assertEqual(
                MTplot_parser(['test', '-n'], test=True)['nodal_line'], True)
            self.assertEqual(
                MTplot_parser(['test', '--nodal_line'], test=True)['nodal_line'], True)
            print('TNP check')
            self.assertEqual(
                MTplot_parser(['test'], test=True)['TNP'], defaults['TNP'])
            self.assertEqual(
                MTplot_parser(['test', '--pt'], test=True)['TNP'], True)
            self.assertEqual(
                MTplot_parser(['test', '--tnp'], test=True)['TNP'], True)
            print('markersize check')
            self.assertEqual(
                MTplot_parser(['test'], test=True)['markersize'], defaults['markersize'])
            self.assertEqual(
                MTplot_parser(['test', '--markersize=8'], test=True)['markersize'], 8)
            self.assertEqual(
                MTplot_parser(['test', '--markersize=8.4'], test=True)['markersize'], 8.4)
            print('station_markersize check')
            self.assertEqual(MTplot_parser(['test'], test=True)[
                             'station_markersize'], defaults['station_markersize'])
            self.assertEqual(MTplot_parser(
                ['test', '--station_markersize=8'], test=True)['station_markersize'], 8)
            self.assertEqual(MTplot_parser(
                ['test', '--station_markersize=8.4'], test=True)['station_markersize'], 8.4)
            print('show_max_likelihood check')
            self.assertEqual(MTplot_parser(['test'], test=True)[
                             'show_max_likelihood'], defaults['show_max_likelihood'])
            self.assertEqual(MTplot_parser(
                ['test', '--showmaxlikelihood'], test=True)['show_max_likelihood'], True)
            self.assertEqual(MTplot_parser(
                ['test', '--show_max_likelihood'], test=True)['show_max_likelihood'], True)
            print('show_mean check')
            self.assertEqual(
                MTplot_parser(['test'], test=True)['show_mean'], defaults['show_mean'])
            self.assertEqual(
                MTplot_parser(['test', '--showmean'], test=True)['show_mean'], True)
            self.assertEqual(
                MTplot_parser(['test', '--show_mean'], test=True)['show_mean'], True)
            print('grid_lines check')
            self.assertEqual(
                MTplot_parser(['test'], test=True)['grid_lines'], defaults['grid_lines'])
            self.assertEqual(
                MTplot_parser(['test', '--gridlines'], test=True)['grid_lines'], True)
            self.assertEqual(
                MTplot_parser(['test', '--grid_lines'], test=True)['grid_lines'], True)
            print('color check')
            self.assertEqual(
                MTplot_parser(['test'], test=True)['color'], defaults['color'])
            self.assertEqual(
                MTplot_parser(['test', '--color=b'], test=True)['color'], 'b')
            self.assertEqual(
                MTplot_parser(['test', '--color=purple'], test=True)['color'], 'purple')
            print('type_label check')
            self.assertEqual(
                MTplot_parser(['test'], test=True)['type_label'], defaults['type_label'])
            self.assertEqual(
                MTplot_parser(['test', '--typelabel'], test=True)['type_label'], True)
            self.assertEqual(
                MTplot_parser(['test', '--type_label'], test=True)['type_label'], True)
            print('hex_bin check')
            self.assertEqual(
                MTplot_parser(['test'], test=True)['hex_bin'], defaults['hex_bin'])
            self.assertEqual(
                MTplot_parser(['test', '--hexbin'], test=True)['hex_bin'], True)
            self.assertEqual(
                MTplot_parser(['test', '--hex_bin'], test=True)['hex_bin'], True)
            print('projection check')
            self.assertEqual(
                MTplot_parser(['test'], test=True)['projection'], defaults['projection'])
            self.assertEqual(
                MTplot_parser(['test', '--projection=uv'], test=True)['projection'], 'uv')
            self.assertEqual(MTplot_parser(
                ['test', '--projection=equalangle'], test=True)['projection'], 'equalangle')
            print('save_file check')
            self.assertEqual(
                MTplot_parser(['test'], test=True)['save_file'], '')
            self.assertEqual(MTplot_parser(
                ['test', '--save_file=test.png'], test=True)['save_file'], 'test.png')
            print('save_dpi check')
            self.assertEqual(
                MTplot_parser(['test'], test=True)['save_dpi'], 200)
            self.assertEqual(
                MTplot_parser(['test', '--save_dpi=150'], test=True)['save_dpi'], 150)
            print('hide')
            self.assertEqual(
                MTplot_parser(['test'], test=True)['show'], True)
            self.assertEqual(
                MTplot_parser(['test', '-q'], test=True)['show'], False)
            self.assertEqual(
                MTplot_parser(['test', '--quiet'], test=True)['show'], False)
            self.assertEqual(
                MTplot_parser(['test', '--hide'], test=True)['show'], False)
            _ARGPARSE = _argparse




"""plot
============================
Submodule for moment tensor plotting.

There are several different plot types available:

    * beachball
    * radiation
    * fault plane
    * hudson
    * lune
    * riedesel jordan

The default format is for 2D plots although 3D plots are possible (dimension=3)

The MTData class provides transparent handling of moment tensor dat
including conversion to different parameters and the calculation of some statistics.
"""

# **Restricted:  For Non-Commercial Use Only**
# This code is protected intellectual property and is available solely for teaching
# and non-commercially funded academic research purposes.
#
# Applications for commercial use should be made to Schlumberger or the University of Cambridge.

import sys
import os

# CHECK DISPLAY
if ('DISPLAY' not in os.environ.keys() and 'win32' not in sys.platform) or (os.environ.get('APPVEYOR', 'False') == 'True'):
    import matplotlib as mpl
    mpl.use('Agg')  # Default no backend

from .core import MTplot, MTData, run, read  # noqa E402, F401



"""
scatangle.py
************
Extension for handling scatangle files (installed with the main module, but provides an example of an extension)
"""


# **Restricted:  For Non-Commercial Use Only**
# This code is protected intellectual property and is available solely for teaching
# and non-commercially funded academic research purposes.
#
# Applications for commercial use should be made to Schlumberger or the University of Cambridge.


import os
import glob
import time
import logging
import operator

import numpy as np

from ..utilities.multiprocessing_helper import JobPool

logger = logging.getLogger('MTfit.extensions.scatangle')

try:
    from . import cscatangle
except ImportError:
    cscatangle = None
except Exception:
    logger.exception('Error importing c extension')
    cscatangle = None

try:
    import argparse  # noqa F401
    _argparse = True  # noqa F811
except Exception:
    _argparse = False


def parse_scatangle(filename, number_location_samples=0, bin_size=0, _use_c=True):
    """
    Read station angles scatter file

    Reads the station angle scatter file. Expected format is given below. TakeOffAngle is 0 down (NED coordinate system).
    The probabilities are read in, so if Oct-tree or metropolis sampling is used, the probability values should be set to one.

    Args
        filename: Angle scatter file name
        location_samples:[Default=0] Number of samples to take from the full location PDF (0 means all samples)
        bin_size:[Default=1] Bin size for sample weighting in degrees (if 0 no binning used) - reduces the number of samples required for well constrained locations.

    Returns
        Records,Probability: Angle Records and the probability for each sample.

    Expected format is:
        Probability
        StationName Azimuth TakeOffAngle
        StationName Azimuth TakeOffAngle

        Probability
        .
        .
        .

    e.g.
        504.7
        S0271   231.1   154.7
        S0649   42.9    109.7
        S0484   21.2    145.4
        S0263   256.4   122.7
        S0142   197.4   137.6
        S0244   229.7   148.1
        S0415   75.6    122.8
        S0065   187.5   126.1
        S0362   85.3    128.2
        S0450   307.5   137.7
        S0534   355.8   138.2
        S0641   14.7    120.2
        S0155   123.5   117
        S0162   231.8   127.5
        S0650   45.9    108.2
        S0195   193.8   147.3
        S0517   53.7    124.2
        S0004   218.4   109.8
        S0588   12.9    128.6
        S0377   325.5   165.3
        S0618   29.4    120.5
        S0347   278.9   149.5
        S0529   326.1   131.7
        S0083   223.7   118.2
        S0595   42.6    117.8
        S0236   253.6   118.6

        502.7
        S0271   233.1   152.7
        S0649   45.9    101.7
        S0484   25.2    141.4
        S0263   258.4   120.7

    """
    # Read file
    with open(filename, 'r') as f:
        station_file = f.readlines()
    multipliers = []
    sample_records = []
    record = {'Name': [], 'Azimuth': [], 'TakeOffAngle': []}
    multiplier = 1.0
    # Loop over lines
    for line in station_file:
        if line.lstrip('\r') == '\n':
            if len(record['Name']) and multiplier:
                record['Azimuth'] = np.matrix(record['Azimuth']).T
                record['TakeOffAngle'] = np.matrix(record['TakeOffAngle']).T
                sample_records.append(record)
                # Using multipliers therefore prob = 1
                multipliers.append(multiplier)
            record = {'Name': [], 'Azimuth': [], 'TakeOffAngle': []}
        elif len(line.rstrip().rstrip('\r').split()) == 1:
            try:
                multiplier = float(line.rstrip().rstrip('\r'))
            except Exception:
                multiplier = 1.0
        else:
            record['Name'].append(line.split()[0])
            record['Azimuth'].append(float(line.split()[1]))
            record['TakeOffAngle'].append(float(line.rstrip().rstrip('\r').split()[2]))
    if len(record['Name']):
        record['Azimuth'] = np.matrix(record['Azimuth']).T
        record['TakeOffAngle'] = np.matrix(record['TakeOffAngle']).T
        sample_records.append(record)
        multipliers.append(multiplier)
    if number_location_samples and number_location_samples < len(sample_records):
        try:
            samples = np.random.choice(len(sample_records), number_location_samples, False)
        except AttributeError:
            i = 0
            samples = np.array(list(set(np.random.randint(0, len(sample_records), len(sample_records)).tolist())))
            while len(samples) < number_location_samples and i < 100:
                samples = np.array(list(set(np.random.randint(0, len(sample_records), len(sample_records)).tolist())))
                i += 1
            if len(samples) < number_location_samples:
                raise ValueError("Couldn't sample angle PDF")
            else:
                samples = samples[:len(sample_records)]
        # Sample randomly from records, probability
        sample_records = list(np.array(sample_records)[samples])
        multipliers = list(np.array(multipliers)[samples])
    new_multipliers = []
    if bin_size:
        old_size = len(sample_records)
        if cscatangle and _use_c:
            logger.info('C code used')
            t0 = time.time()
            sample_records, multipliers = cscatangle.bin_scatangle(sample_records, np.array(multipliers), bin_size)
            logger.info('Elapsed time = {}'.format(time.time()-t0))
        else:
            logger.info('Python code used')
            t0 = time.time()
            for i, record in enumerate(sample_records):
                if not (i+1) % 10:
                    logger.info('{} records completed'.format(i+1))
                j = i+1
                multiplier = multipliers[i]
                while j < len(sample_records):
                    toa_diff = sample_records[j]['TakeOffAngle']-record['TakeOffAngle']
                    az_diff = sample_records[j]['Azimuth']-record['Azimuth']
                    if np.max(np.abs(toa_diff)) < bin_size/2.0 and np.max(np.abs(az_diff)) < bin_size/2.0:
                        multiplier += multipliers.pop(j)
                        sample_records.pop(j)
                    else:
                        j += 1
                new_multipliers.append(multiplier)
            multipliers = new_multipliers
            logger.info('Elapsed time = {}'.format(time.time()-t0))
        logger.info('{} degree binning reduced {}  samples to {} samples.'.format(bin_size, old_size, len(sample_records)))
    return sample_records, multipliers


def _output_scatangle(filename, samples, probabilities):
    """
    Output scatangle file from samples and probabilities.

    Args
        filename: str name of file to output to.
        samples: list of location samples.
        probabilities: list of sample probabilities.
    """
    output = []
    # Loop over samples
    for i, sample in enumerate(samples):
        # Append multiplier (probability)
        output.append(str(probabilities[i]))
        # Loop over stations
        for j, st in enumerate(sample['Name']):
            output.append(st+'\t'+str(float(sample['Azimuth'][j]))+'\t'+str(float(sample['TakeOffAngle'][j])))
        output.append('')
    with open(filename, 'w') as f:
        f.write('\n'.join(output))


def bin_scatangle(filename, number_location_samples=0, bin_size=1):
    """
    Bin scatangle samples into bins of size given by the bin size argument, if all the differences in angles for each station between the two samples are within that range

    Args
        filename: str of filename to read.
        number_location_samples:[0} integer number of location samples to sub-sample (0 means to use all).
        bin_size:[1.0] float size of bin to stack samples over.
    """
    # Parse scatangle and bin
    sample_records, multipliers = parse_scatangle(filename, number_location_samples, bin_size)
    old_filename = filename
    # Add _bin_ to filename
    filename = ('_bin_'+str(bin_size)).join(os.path.splitext(filename))
    # Output to disk
    _output_scatangle(filename, sample_records, multipliers)
    return old_filename, filename


class BinScatangleTask(object):
    """
    Scatangle Binning task

    Bins and Saves scatangle file

    Initialisation
        Args
            fid: Filename for MATLAB output.
            number_location_samples: number_location_samples.
            bin_size: bin size.
    """

    def __init__(self, fid, number_location_samples=0, bin_size=1.0):
        """
        Initialisation of MatlabOutputTask

        Args
            fid: Filename for MATLAB output.
            output: Dictionary of output to be saved to fid.

        """
        self.fid = fid
        self.number_location_samples = number_location_samples
        self.bin_size = bin_size

    def __call__(self):
        """
        Runs the MATLAB output task

        Runs the MatlabOutputTask and returns a result code.

        Returns
            resultCode: 10 if successful, 20 if an exception is thrown.

        """
        try:
            return bin_scatangle(self.fid, self.number_location_samples, self.bin_size)
        except Exception:
            logger.exception('Scatangle Bin Error')
            return 20


def bin_scatangle_files(files, number_location_samples=0, bin_scatangle_size=1.0, parallel=True, mpi=False, **kwargs):
    """
    Bin scatangle samples into bins of size given by the bin size argument,
    if all the differences in angles for each station between the two samples are within that range

    Args
        filename: str of filename to read.
        number_location_samples:[0} integer number of location samples to sub-sample (0 means to use all).
        bin_scatangle_size:[1.0] float size of bin to stack samples over.
        parallel:[True] boolean to run in parallel using job pool (overridden by mpi option).
        mpi: [False] boolean to run using MPI (ignores parallel flag).

    Returns
        new_files:list of new file names
    """
    # check path and glob
    if not isinstance(files, list) and os.path.isdir(files):
        files = glob.glob('*.'+kwargs['angle_extension'])
    elif not isinstance(files, list) and '*' in files:
        files = glob.glob(files)
    # Make sure the files are a list
    if not isinstance(files, list):
        files = [files]
    # Set parallel flag
    parallel = parallel and not len(files) == 1
    # Get MPI parameters if running using MPI
    if mpi:
        try:
            from mpi4py import MPI
            comm = MPI.COMM_WORLD
        except Exception:
            mpi = False
    # Otherwise create jobpool if running in parallel
    elif parallel:
        job_pool = JobPool(task=BinScatangleTask)
        nworkers = len(job_pool)
        if nworkers == 1:
            job_pool.close()
            parallel = False
    new_files = files[:]
    if mpi:
        # Sort files into size lists
        data = [[] for i in range(comm.Get_size())]
        j = 0
        for fn in files:
            data[j].append(fn)
            j = (j+1) % comm.Get_size()
        # Loop over files for each process
        for fn in data[comm.Get_rank()]:
            # If the filename is not blank, then bin it.
            if len(fn):
                new_files[new_files.index(fn)] = bin_scatangle(fn, number_location_samples, bin_scatangle_size)[1]
        end = True
        # Wait for end
        items = comm.gather(end, 0)
        if comm.Get_rank() == 0:
            for i in items:
                assert i is True
    elif parallel:
        # Loop over pool running tasks
        for fn in files:
            job_pool.task(fn, number_location_samples, bin_scatangle_size)
        # Get results
        results = job_pool.all_results()
        for new_fn in results:
            new_files[new_files.index(new_fn[0])] = new_fn[1]
        job_pool.close()
    else:
        # Run bin for each file
        for fn in files:
            new_files[new_files.index(fn)] = bin_scatangle(fn, number_location_samples, bin_scatangle_size)[1]
    return new_files


def convert_scatangle_to_MATLAB(scatangle_file, fid=False, data_file=False):
    """
    Converts scatangle file to MATLAB station distribution format

    Converts the scatangle file to the station distribution format separately from the inversion.

    Args
        scatangle_file: Scatangle filename.
        fid:[False] Output filename.
        data_file:[False] Data file path (means that only stations with observations are outputted).
    """
    from ..inversion import Inversion
    # Parse scatangle file
    X = parse_scatangle(scatangle_file)
    if not fid:
        fid = scatangle_file
    if data_file:
        # Check Stations for only those with observations
        data = Inversion({'UID': 1}, parallel=False)._load(data_file)
        original_samples = [u.copy() for u in X[0]]
        stations = []
        for key in [u for u in data.keys() if 'amplituderatio' in u.lower() or 'amplitude_ratio' in u.lower() or 'polarity' in u.lower()]:
            stations.extend(data[key]['Stations']['Name'])
        stations_set = set(stations)
        location_samples = [u.copy() for u in original_samples]
        selected_stations = list(set(location_samples[0]['Name']) & stations_set)
        indices = [location_samples[0]['Name'].index(u) for u in selected_stations]
        for i, sample in enumerate(location_samples):
            sample['Name'] = operator.itemgetter(*indices)(sample['Name'])
            sample['TakeOffAngle'] = sample['TakeOffAngle'][indices]
            sample['Azimuth'] = sample['Azimuth'][indices]
        # Output results
        Inversion(data_file=data_file, parallel=False).__output__({}, fid=fid, location_samples=location_samples,
                                                                  location_sample_multipliers=X[1], station_only=True)
    else:
        # Output results
        Inversion({'UID': 1}, parallel=False).__output__({}, fid=fid, location_samples=X[0],
                                                         location_sample_multipliers=X[1], station_only=True)


def parser_check(parser, options, defaults):
    flags = []
    if options['bin_scatangle']:
        if not options['location_pdf_file_path']:
            options['location_pdf_file_path'] = glob.glob(options['data_file']+os.path.sep+'*'+options['angle_extension'])
        if not isinstance(options['location_pdf_file_path'], list):
            options['location_pdf_file_path'] = [options['location_pdf_file_path']]
        flags = ['no_location_update']
    return options, flags


# MTfit pkg_resources EntryPoint functions


PARSER_DEFAULTS = {
          'bin_scatangle': False,
          'bin_size': 1.0,
          }
PARSER_DEFAULT_TYPES = {'bin_scatangle': [bool], 'bin_size': [float]}


def cmd_defaults():
    return(PARSER_DEFAULTS, PARSER_DEFAULT_TYPES)


def cmd_opts(group, argparser=_argparse, defaults=PARSER_DEFAULTS):
    """
    Adds parser group for scatangle arguments

    Returns
        group: argparse or optparse argument group

    """
    if argparser:
        group.add_argument("--bin-scatangle", "--binscatangle", "--bin_scatangle",  action="store_true",
                           default=defaults['bin_scatangle'], help="Bin the scatangle file to reduce the number of samples [default=False]. --bin-size Sets the bin size parameter .",
                           dest="bin_scatangle")
        group.add_argument("--bin-size", "--binsize", "--bin_size", type=float, default=defaults['bin_size'],
                           help="Sets the scatangle bin size parameter [default={}]".format(defaults['bin_size']), dest="bin_scatangle_size")
    else:
        group.add_option("--bin-scatangle", "--binscatangle", "--bin_scatangle",  action="store_true",
                         default=defaults['bin_scatangle'], help="Bin the scatangle file to reduce the number of samples [default=False]. --bin-size Sets the bin size parameter .",
                         dest="bin_scatangle")
        group.add_option("--bin-size", "--binsize", "--bin_size", type=float, default=defaults['bin_size'],
                         help="Sets the scatangle bin size parameter [default={}]".format(defaults['bin_size']), dest="bin_scatangle_size")
    return group, parser_check


def pre_inversion(**kwargs):
    if kwargs.get('bin_scatangle', False):
        try:
            kwargs['location_pdf_file_path'] = bin_scatangle_files(kwargs.get('location_pdf_file_path'), **kwargs)
            kwargs.pop('number_location_samples')
        except Exception:
            pass
    return kwargs


def location_pdf_parser(*args, **kwargs):
    return parse_scatangle(*args, **kwargs)



import time


class TaskTest(object):

    def __init__(self, a, b):
        self.a = a
        self.b = b

    def __call__(self):
        print('Test 1 {}'.format(self.a*self.b))
        return self.a*self.b


class TaskTest2(object):

    def __init__(self, a, b):
        self.a = a
        self.b = b

    def __call__(self):
        print('Test 2 {}'.format(self.a+self.b))
        time.sleep(self.b*0.5)
        return self.a+self.b



# **Restricted:  For Non-Commercial Use Only**
# This code is protected intellectual property and is available solely for teaching
# and non-commercially funded academic research purposes.
#
# Applications for commercial use should be made to Schlumberger or the University of Cambridge.


import os


from .plot_classes import MTplot, MTData  # noqa F401
from ..utilities.file_io import read_matlab_output
from ..utilities.file_io import read_pickle_output
from ..utilities.file_io import read_hyp_output
from ..utilities.file_io import read_scatangle_output
from ..utilities.argparser import MTplot_parser as _parser
from ..utilities.extensions import get_extensions


def read(filename, station_distribution=False, *args, **kwargs):
    """
    Read filename or list of filenames (can be nested for multiplot MTplot call)

    Args
        filename: str/list filenames to read

    Returns
        list/dict, list/dict: tuple of event and station data
    """
    if type(filename) == list:
        events = []
        stations = []
        for fn in filename:
            if station_distribution:
                stations.append(read(fn))
            else:
                # recursive to allow for nested list structure for multiplot
                ev, st = read(fn)
                events.append(ev)
                stations.append(st)
    elif station_distribution:
        return _read(filename, station_distribution=station_distribution, *args, **kwargs)
    else:
        events, stations = _read(filename)
    if station_distribution:
        return stations
    return events, stations


def _read(filename, file_format=False, station_distribution=False, *args, **kwargs):
    """
    Read filename

    Args
        filename: str filenames to read

    Keyword Args
        file_format: str file format (default is to try and use ending)

    Returns
        dict, dict: tuple of event and station data
    """
    default_formats = {'.hyp': read_hyp_output, '.mt': read_hyp_output, '.mat': read_matlab_output,
                       '.out': read_pickle_output, '.pkl': read_pickle_output, '.scatangle': read_scatangle_output}
    parser_names, parsers = get_extensions('MTfit.plot_read', default_formats)
    if not file_format:
        file_format = os.path.splitext(filename)[-1]
    return parsers[file_format](filename, station_distribution=station_distribution, *args, **kwargs)


def plot(filename, *args, **kwargs):
    """
    Plot the filename using MTplot

    Args
        filename: str/list filenames to read
        kwargs: passed to MTplot call
        args: passed to MTplot call

    Returns
        MTplot object

    """
    # Handle File IO
    MTs, stations = read(filename)
    # Plot results
    return MTplot(MTs, stations=stations, *args, **kwargs)


def run(input_args=[]):
    """
    Run from the command line

    Has a blocking plot.show call, so will wait until the plot is closed.

    Keyword Args:
        input_args: list of input arguments

    Returns
        MTplot object
    """
    options = _parser(input_args)
    filename = options.pop('data_file')
    h = plot(filename, **options)
    return h



import unittest

import numpy as np

from MTfit.plot.plot_classes import MTData
from MTfit.utilities.unittest_utils import TestCase


class MTDataTestCase(TestCase):

    def setUp(self):
        self.MTData = MTData(np.array([[1, 2, 0, 0, 1, 0, 2, -1],
                                       [0, 0, 0, 0, 0, 0, 1, 0],
                                       [-1, -1, 0, 0, -1,
                                           0, 1, 3],
                                       [0, 0, 1, 0, 0, 1, 0, 1],
                                       [0, 0, 0, 0, 0, 0, 0, 0],
                                       [0, 0, 0, 1, 0, 0, 0, 0]]))

    def tearDown(self):
        del self.MTData

    @unittest.expectedFailure
    def test___init__(self):
        raise NotImplementedError()

    @unittest.expectedFailure
    def test__set_ln_pdf(self):
        raise NotImplementedError()

    @unittest.expectedFailure
    def test__clear_dependent_parameters(self):
        raise NotImplementedError()

    @unittest.expectedFailure
    def test___eq__(self):
        raise NotImplementedError()

    @unittest.expectedFailure
    def test__del_converted(self):
        raise NotImplementedError()

    @unittest.expectedFailure
    def test__set_converted(self):
        raise NotImplementedError()

    @unittest.expectedFailure
    def test_is_dc(self):
        raise NotImplementedError()

    def test__set_probability(self):
        self.MTData._set_probability([1])
        self.assertEqual(len(self.MTData.probability), 1)
        with self.assertRaises(ValueError):
            self.MTData._set_probability([1, 2])
        self.MTData._set_probability([1, 2, 3, 4, 5, 6, 7, 8])
        self.assertEqual(len(self.MTData.probability), 8)

    def test__check(self):
        new_mts = self.MTData._check(
            np.matrix([[1, 0, 0], [0, 0, 0], [0, 0, -1]]))
        self.assertEqual(new_mts.shape[0], 6)
        self.assertEqual(new_mts.shape[1], 1)
        new_mts = self.MTData._check(np.array([[[1, 0, 0], [0, 0, 0], [
                                     0, 0, -1]], [[1, 0, 0], [0, 0, 0], [0, 0, -1]], [[1, 0, 0], [0, 0, 0], [0, 0, -1]]]))
        self.assertEqual(new_mts.shape[0], 6)
        self.assertEqual(new_mts.shape[1], 3)

    def test___getitem__(self):
        newMTData = self.MTData[:, 0:3]
        self.assertTrue(isinstance(newMTData, MTData))
        self.assertEqual(len(newMTData), 3)
        self.assertEqual(len(self.MTData), 8)
        self.assertEqual(newMTData.strike[0], self.MTData.strike[0])
        self.MTData.u[0]
        newMTData = self.MTData[:, 0:4]
        self.assertTrue(isinstance(newMTData, MTData))
        self.assertEqual(len(newMTData), 4)
        self.assertTrue('u' in newMTData.__dict__.keys() and 'strike' in newMTData.__dict__.keys())
        self.MTData.mean_orientation
        newMTData = self.MTData[:, 0:4]
        self.assertFalse('mean_strike' in newMTData.__dict__.keys() or 'mean_dip' in newMTData.__dict__.keys() or 'mean_rake' in newMTData.__dict__.keys() or
                         'mean_normal' in newMTData.__dict__.keys() or 'var_clustered_rake1' in newMTData.__dict__.keys() or 'cov_clustered_N1' in newMTData.__dict__.keys())

    def test___setitem__(self):
        og = self.MTData.gamma[0]
        self.MTData[:, 0] = np.array([0, 1, 0, -1, -2, 0])
        self.assertEqual(len(self.MTData), 8)
        self.assertNotEqual(self.MTData.gamma[0], og)
        self.assertTrue((self.MTData[:, 0] == np.array([[0, 1, 0, -1, -2, 0]]).T))

    def test___delitem__(self):
        with self.assertRaises(ValueError):
            del self.MTData[:, 0]
        return
        # Not carried out
        og = self.MTData.gamma[0]
        del self.MTData[:, 0]
        self.assertEqual(len(self.MTData), 7)
        self.assertNotEqual(self.MTData.gamma[0], og)
        self.assertTrue(
            (self.MTData[:, 0] == np.array([[2, 0, -1, 0, 0, 0]]).T).all())

    def test___len__(self):
        self.assertEqual(len(self.MTData), 8)
        self.assertEqual(len(self.MTData), self.MTData.MTs.shape[1])

    def test___str__(self):
        self.assertEqual(self.MTData.__str__(), self.MTData.MTs.__str__())

    def test___repr__(self):
        self.assertEqual(self.MTData.__repr__(), self.MTData.MTs.__repr__())

    def test__getattr__(self):
        with self.assertRaises(AttributeError):
            self.MTData.test
        self.assertEqual(self.MTData.gamma[0], 0)
        self.assertEqual(self.MTData.u[0], 0)
        try:
            self.assertEqual(self.MTData.strike[0], 90)
        except Exception:
            self.assertEqual(self.MTData.strike[0], 270)

    def test__get_converted(self):
        self.assertEqual(self.MTData._get_converted(), {})
        self.MTData.gamma
        self.assertEqual(tuple(sorted(self.MTData._get_converted().keys())),
                         ('E', 'N', 'P', 'T', 'delta', 'gamma'))
        self.assertEqual(tuple(sorted(self.MTData._get_converted((slice(None, None, None), 2)).keys())),
                         ('E', 'N', 'P', 'T', 'delta', 'gamma'))
        self.assertAlmostEqual(self.MTData._get_converted((slice(None, None, None), 2))['delta'], 0.0)
        self.assertEqual(self.MTData._get_converted((slice(None, None, None), 2))['E'].shape, (3,))
        self.assertEqual(self.MTData._get_converted((slice(None, None, None), 2))['T'].shape, (3,))

    def test__convert(self):
        test_data = [('gamma', 0.), ('delta', 0.), ('E', np.array([[1.], [0], [-1.]])), ('T', np.matrix([[1], [0], [0]])),
                     ('N', np.matrix([[0], [1], [0]])), ('P', np.matrix(
                         [[0], [0], [1]])), ('u', 0.), ('v', 0.),
                     ('tau', 0.), ('k', 0.), ('strike1', 90), ('dip1',
                                                               45), ('rake1', -90), ('strike2', 270), ('dip2', 45),
                     ('rake2', -90), ('strike', 90), ('dip', 45), ('rake', -
                                                                   90), ('N1', np.matrix([[1/np.sqrt(2)], [0], [1/np.sqrt(2)]])),
                     ('N2', np.matrix([[1/np.sqrt(2)], [0], [-1/np.sqrt(2)]])), ('kappa', np.pi/2), ('h', np.cos(np.pi/4)),
                     ('sigma', -np.pi/2), ('phi1', np.array([[1/np.sqrt(2)], [0.], [1/np.sqrt(2)]])),
                     ('phi2', np.array([[1/np.sqrt(2)], [0.], [-1/np.sqrt(2)]])), ('area_displacement', 1.0),
                     ('explosion', 0.0)]
        for attr, value in test_data:
            if attr in ['T', 'N', 'P', 'E', 'N1', 'N2']:
                self.assertTrue((self.MTData[:, 0]._convert(attr) == value).all())
            else:
                if attr in ['strike1', 'strike2', 'dip1', 'dip2', 'rake1', 'rake2', 'strike', 'dip', 'rake']:
                    try:
                        self.assertAlmostEquals(self.MTData[:, 0]._convert(attr), value, 10)
                    except AssertionError:
                        if '1' in attr:
                            alt_attr = attr.replace('1', '2')
                        else:
                            alt_attr = attr.replace('2', '1')
                        if alt_attr[-1] not in ['1', '2']:
                            alt_attr += '2'
                        value = dict(test_data)[alt_attr]
                        self.assertAlmostEquals(self.MTData[:, 0]._convert(attr), value, 10)
                else:
                    self.assertAlmostEquals(self.MTData[:, 0]._convert(attr), value, 10)
        self.MTData._set_ln_pdf([0.1, 0.2, 0.3, 0.2, 0.1, 0.4, 0.6, 0.8])
        self.MTData.total_number_samples = 10000
        self.assertAlmostEquals(self.MTData.ln_bayesian_evidence, -6.764305312122505)

    def test_cluster_normals(self):
        self.tearDown()
        self.MTData = MTData(np.array([[1, 0.9,    1,    1, 0.8, 1.1, 0.9,    1],
                                       [0,   0,    0,    0,   0,   0,   0,    0],
                                       [-1, -1, -0.9, -0.8,  -1,  -1,  -1, -0.9],
                                       [0,   0,  0.1,    0, 0.2,   0,   0,    0],
                                       [0, 0.1,    0,    0,   0,   0,   0,    0],
                                       [0,   0,    0,  0.2,   0,   0,   0,    0]]))
        self.MTData.cluster_normals()
        self.assertTrue((self.MTData.clustered_N1[2, :] < 0).all())
        self.assertTrue((self.MTData.clustered_N2[2, :] > 0).all())

    def test_get_mean_orientation(self):
        self.test_cluster_normals()
        [s, d, r] = self.MTData.mean_orientation
        self.assertAlmostEquals(s, 272.91522545)
        self.assertAlmostEquals(d, 45.2947782)
        self.assertAlmostEquals(r, -88.295161590065447)
        self.assertAlmostEquals(self.MTData.var_clustered_rake1, 23.251792031905531)
        self.assertTrue(self.MTData.cov_clustered_N1.max() < 0.005)
        self.MTData._set_probability([0.05, 0.15, 0.1, 0.3, 0.1, 0.1, 0.1, 0.1])
        [s, d, r] = self.MTData.mean_orientation
        self.assertAlmostEquals(s, 274.25373043)
        self.assertAlmostEquals(d, 45.47487435)
        self.assertAlmostEquals(r, -85.908387816157074)
        self.assertAlmostEquals(self.MTData.var_clustered_rake1, 46.782048639043481)
        self.assertTrue(self.MTData.cov_clustered_N1.max() < 0.007)

    def test_get_mean(self):
        self.test_cluster_normals()
        mts = self.MTData.mean
        mean = np.array([[0.9625],
                         [0.],
                         [-0.95],
                         [0.0375],
                         [0.0125],
                         [0.025]])
        covariance = np.array([[0.00839286,  0.,  0.00214286, -0.00410714, -0.00089286,
                                0.00107143],
                               [0.,  0.,  0.,  0.,  0.,
                                0.],
                               [0.00214286,  0.,  0.00571429, -0.00071429, -0.00071429,
                                0.00428571],
                               [-0.00410714,  0., -0.00071429,  0.00553571, -0.00053571,
                                -0.00107143],
                               [-0.00089286,  0., -0.00071429, -0.00053571,  0.00125,
                                -0.00035714],
                               [0.00107143,  0.,  0.00428571, -0.00107143, -0.00035714,
                                0.005]])
        self.assertAlmostEquals(mts, mean)
        cov = self.MTData.covariance
        self.assertAlmostEquals(cov, covariance)
        self.MTData._set_probability(
            [0.05, 0.15, 0.1, 0.3, 0.1, 0.1, 0.1, 0.1])
        mean = np.array([[0.965],
                         [0.],
                         [-0.92],
                         [0.03],
                         [0.015],
                         [0.06]])
        mts = self.MTData.mean
        self.assertAlmostEquals(mts, mean)
        covariance = np.matrix([[0.00751497,  0.,  0.00335329, -0.00353293, -0.00116766,
                                 0.00251497],
                                [0.,  0.,  0.,  0.,  0.,
                                 0.],
                                [0.00335329,  0.,  0.0091018, -0.00167665, -0.00143713,
                                 0.00862275],
                                [-0.00353293,  0., -0.00167665,  0.00491018, -0.00053892,
                                 -0.00215569],
                                [-0.00116766,  0., -0.00143713, -0.00053892,  0.00152695,
                                 -0.00107784],
                                [0.00251497,  0.,  0.00862275, -0.00215569, -0.00107784,
                                 0.01005988]])
        cov = self.MTData.covariance
        self.assertAlmostEquals(cov, covariance)

    def test_get_max_probability(self):
        self.test_cluster_normals()
        self.MTData._set_probability(
            [0.05, 0.15, 0.1, 0.3, 0.1, 0.1, 0.1, 0.1])
        max_prob = np.matrix([[1],
                              [0],
                              [-0.8],
                              [0],
                              [0],
                              [0.2]])
        max_p = self.MTData.max_probability
        self.assertAlmostEquals(max_p.MTs, max_prob)
        self.assertAlmostEquals(max_p.MTs, self.MTData.MTs[
                                :, self.MTData.probability == self.MTData.probability.max()])
        self.MTData._set_probability(
            [0.05, 0.15, 0.1, 0.3, 0.1, 0.3, 0.1, 0.1])
        max_prob = np.matrix([[1, 1.1],
                              [0,  0],
                              [-0.8, -1],
                              [0,  0],
                              [0,  0],
                              [0.2,  0]])
        max_p = self.MTData.max_probability
        self.assertAlmostEquals(max_p.MTs, max_prob)
        self.assertAlmostEquals(max_p.MTs, self.MTData.MTs[
                                :, self.MTData.probability == self.MTData.probability.max()])
        max_prob = np.matrix([[1],
                              [0],
                              [-0.8],
                              [0],
                              [0],
                              [0.2]])
        max_p = self.MTData.get_max_probability(single=True)
        self.assertAlmostEquals(max_p.MTs, max_prob)

    def test_get_unique_McMC(self):
        self.tearDown()
        self.MTData = MTData(np.array([[1, 2, 0, 0, 1, 0, 2, -1],
                                       [0, 0, 0, 0, 0, 0, 0, 0],
                                       [-1, -1, 0, 0, -1,
                                       0, -1, 3],
                                       [0, 0, 1, 1, 0, 1, 0, 1],
                                       [0, 0, 0, 0, 0, 0, 0, 0],
                                       [0, 0, 0, 0, 0, 0, 0, 0]]))
        newMTData = self.MTData.get_unique_McMC()
        self.assertEqual(len(newMTData), 4)
        self.assertEqual(set(newMTData.probability), set([2, 2, 3, 1]))
        self.tearDown()
        self.MTData = MTData(np.array([[1, 2, 0, 0, 1, 0, 1, -1],
                                       [0, 0, 0, 0, 0, 0, 0, 0],
                                       [-1, -1, 0, 0, -1,
                                       0, -1, 3],
                                       [0, 0, 1, 1, 0, 1, 0, 1],
                                       [0, 0, 0, 0, 0, 0, 0, 0],
                                       [0, 0, 0, 0, 0, 0, 0, 0]]))
        newMTData = self.MTData.get_unique_McMC()
        self.assertEqual(len(newMTData), 4)
        self.assertEqual(set(newMTData.probability), set([3, 1, 3, 1]))
        self.MTData.gamma
        newMTData = self.MTData.get_unique_McMC()
        self.assertEqual(len(newMTData), 4)
        self.assertEqual(set(newMTData.probability), set([3, 1, 3, 1]))
        self.assertEqual(len(newMTData.gamma), 4)
        self.assertIn(0., newMTData.gamma)

    def test_MTindices(self):
        self.assertAlmostEquals(self.MTData.xx, np.array([1, 2, 0, 0, 1, 0, 2, -1]))
        self.assertAlmostEquals(self.MTData.yy, np.array([0, 0, 0, 0, 0, 0, 1, 0]))
        self.assertAlmostEquals(self.MTData.zz, np.array([-1, -1, 0, 0, -1, 0, 1, 3]))
        self.assertAlmostEquals(self.MTData.xy, np.array([0, 0, 1, 0, 0, 1, 0, 1]))
        self.assertAlmostEquals(self.MTData.xz, np.array([0, 0, 0, 0, 0, 0, 0, 0]))
        self.assertAlmostEquals(self.MTData.yz, np.array([0, 0, 0, 1, 0, 0, 0, 0]))


class MTplotTestCase(TestCase):

    @unittest.expectedFailure
    def test___init__(self):
        raise NotImplementedError()

    @unittest.expectedFailure
    def test__prep_data(self):
        raise NotImplementedError()

    @unittest.expectedFailure
    def test_plot(self):
        raise NotImplementedError()

    @unittest.expectedFailure
    def test_ax_labels(self):
        raise NotImplementedError()


class _BasePlotTestCase(TestCase):

    @unittest.expectedFailure
    def test___init__(self):
        raise NotImplementedError()

    @unittest.expectedFailure
    def test___call__(self):
        raise NotImplementedError()

    @unittest.expectedFailure
    def test_plot(self):
        raise NotImplementedError()

    @unittest.expectedFailure
    def test__convert(self):
        raise NotImplementedError()

    @unittest.expectedFailure
    def test__convert_mts(self):
        raise NotImplementedError()

    @unittest.expectedFailure
    def test__ax_plot(self):
        raise NotImplementedError()

    @unittest.expectedFailure
    def test__background(self):
        raise NotImplementedError()

    @unittest.expectedFailure
    def test__eigenvector_matrix(self):
        raise NotImplementedError()

    @unittest.expectedFailure
    def test__surf_plot(self):
        raise NotImplementedError()

    @unittest.expectedFailure
    def test__scatter_plot(self):
        raise NotImplementedError()

    @unittest.expectedFailure
    def test__line_plot(self):
        raise NotImplementedError()

    @unittest.expectedFailure
    def test__text(self):
        raise NotImplementedError()

    @unittest.expectedFailure
    def test__2d_surf_plot(self):
        raise NotImplementedError()

    @unittest.expectedFailure
    def test__2d_scatter_plot(self):
        raise NotImplementedError()

    @unittest.expectedFailure
    def test__2d_line_plot(self):
        raise NotImplementedError()

    @unittest.expectedFailure
    def test__2d_text(self):
        raise NotImplementedError()

    @unittest.expectedFailure
    def test__3d_surf_plot(self):
        raise NotImplementedError()

    @unittest.expectedFailure
    def test__3d_scatter_plot(self):
        raise NotImplementedError()

    @unittest.expectedFailure
    def test__3d_line_plot(self):
        raise NotImplementedError()

    @unittest.expectedFailure
    def test__3d_text(self):
        raise NotImplementedError()


class _FocalSpherePlotTestCase(TestCase):

    @unittest.expectedFailure
    def test__init__(self):
        raise NotImplementedError()

    @unittest.expectedFailure
    def test__convert_mts(self):
        raise NotImplementedError()

    @unittest.expectedFailure
    def test__ax_plot(self):
        raise NotImplementedError()

    @unittest.expectedFailure
    def test_plot_plane(self):
        raise NotImplementedError()

    @unittest.expectedFailure
    def test__get_great_circle(self):
        raise NotImplementedError()

    @unittest.expectedFailure
    def test__get_small_circle(self):
        raise NotImplementedError()

    @unittest.expectedFailure
    def test__rotation_matrix(self):
        raise NotImplementedError()

    @unittest.expectedFailure
    def test__get_nodal_line(self):
        raise NotImplementedError()

    @unittest.expectedFailure
    def test__axis_lines(self):
        raise NotImplementedError()

    @unittest.expectedFailure
    def test__boundary_lines(self):
        raise NotImplementedError()

    @unittest.expectedFailure
    def test__fault_plane(self):
        raise NotImplementedError()

    @unittest.expectedFailure
    def test__plot_TNP(self):
        raise NotImplementedError()

    @unittest.expectedFailure
    def test__nodal_line(self):
        raise NotImplementedError()

    @unittest.expectedFailure
    def test__background(self):
        raise NotImplementedError()

    @unittest.expectedFailure
    def test__stations(self):
        raise NotImplementedError()

    @unittest.expectedFailure
    def test__plot_stations(self):
        raise NotImplementedError()


class _AmplitudePlotTestCase(TestCase):

    @unittest.expectedFailure
    def test__convert_mts(self):
        raise NotImplementedError()

    @unittest.expectedFailure
    def test__ax_plot(self):
        raise NotImplementedError()

    @unittest.expectedFailure
    def test__background(self):
        raise NotImplementedError()


class _RadiationPlotTestCase(TestCase):

    @unittest.expectedFailure
    def test__convert_mts(self):
        raise NotImplementedError()

    @unittest.expectedFailure
    def test__boundary_lines(self):
        raise NotImplementedError()

    @unittest.expectedFailure
    def test__stations(self):
        raise NotImplementedError()


class _FaultPlanePlotTestCase(TestCase):

    @unittest.expectedFailure
    def test___init__(self):
        raise NotImplementedError()

    @unittest.expectedFailure
    def test__convert_mts(self):
        raise NotImplementedError()

    @unittest.expectedFailure
    def test__ax_plot(self):
        raise NotImplementedError()

    @unittest.expectedFailure
    def test__background(self):
        raise NotImplementedError()


class _HistPlotTestCase(TestCase):

    @unittest.expectedFailure
    def test___init__(self):
        raise NotImplementedError()

    @unittest.expectedFailure
    def test__ax_plot(self):
        raise NotImplementedError()

    @unittest.expectedFailure
    def test__hist(self):
        raise NotImplementedError()

    @unittest.expectedFailure
    def test__max_2d_for_hist(self):
        raise NotImplementedError()

    @unittest.expectedFailure
    def test__2d_hist(self):
        raise NotImplementedError()

    @unittest.expectedFailure
    def test__background(self):
        raise NotImplementedError()


class _LunePlotTestCase(TestCase):

    @unittest.expectedFailure
    def test__convert_mts(self):
        raise NotImplementedError()

    @unittest.expectedFailure
    def test__ax_plot(self):
        raise NotImplementedError()

    @unittest.expectedFailure
    def test__lune_coords(self):
        raise NotImplementedError()

    @unittest.expectedFailure
    def test__2d_hist(self):
        raise NotImplementedError()

    @unittest.expectedFailure
    def test__get_small_circle(self):
        raise NotImplementedError()

    @unittest.expectedFailure
    def test__boundary_lines(self):
        raise NotImplementedError()

    @unittest.expectedFailure
    def test__stations(self):
        raise NotImplementedError()

    @unittest.expectedFailure
    def test__plot_TNP(self):
        raise NotImplementedError()

    @unittest.expectedFailure
    def test__nodal_line(self):
        raise NotImplementedError()


class _HudsonPlotTestCase(TestCase):

    @unittest.expectedFailure
    def test___init__(self):
        raise NotImplementedError()

    @unittest.expectedFailure
    def test__convert_mts(self):
        raise NotImplementedError()

    @unittest.expectedFailure
    def test__ax_plot(self):
        raise NotImplementedError()

    @unittest.expectedFailure
    def test__background(self):
        raise NotImplementedError()

    @unittest.expectedFailure
    def test__boundary_lines(self):
        raise NotImplementedError()

    @unittest.expectedFailure
    def test__uv_boundary_lines(self):
        raise NotImplementedError()

    @unittest.expectedFailure
    def test__tk_boundary_lines(self):
        raise NotImplementedError()


class _RiedeselJordanPlotTestCase(TestCase):

    @unittest.expectedFailure
    def test___init__(self):
        raise NotImplementedError()

    @unittest.expectedFailure
    def test__convert_mts(self):
        raise NotImplementedError()

    @unittest.expectedFailure
    def test__ax_plot(self):
        raise NotImplementedError()

    @unittest.expectedFailure
    def test__background(self):
        raise NotImplementedError()

    @unittest.expectedFailure
    def test__stations(self):
        raise NotImplementedError()

    @unittest.expectedFailure
    def test__boundary_lines(self):
        raise NotImplementedError()


class _ParameterHistPlotTestCase(TestCase):

    @unittest.expectedFailure
    def test___init__(self):
        raise NotImplementedError()

    @unittest.expectedFailure
    def test__ax_plot(self):
        raise NotImplementedError()

    @unittest.expectedFailure
    def test__convert(self):
        raise NotImplementedError()

    @unittest.expectedFailure
    def test__background(self):
        raise NotImplementedError()


class _TapePlotTestCase(TestCase):

    @unittest.expectedFailure
    def test___init__(self):
        raise NotImplementedError()

    @unittest.expectedFailure
    def test_plot(self):
        raise NotImplementedError()
